def check_duplicate_stats(stats1, stats2, threshold = 0.01):
    '''
    Checks two lists of paired statistics for duplicates and returns a list of
    the pairs that agree within to <1%.

    INPUTS:
    STATS1 : List of first statistical metric, e.g. Standard Deviations
    STATS2 : List of second statistical metric, e.g. Centered Root Mean Square Difference

    OUTPUTS:
    DUPLICATES : List of tuples of paired statistics that are duplicates. The list contains
                 the index locations of the pairs of statistics followed by their values
                 as 2-tuples.

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Apr 23, 2017
    '''
    if threshold < 1e-7:
        ValueError('threshold value must be positive: ' + str(threshold))

    # Check for non-empty lists
    if len(stats1) == 0:
        ValueError('Argument stats1 is empty list!')
    elif len(stats2) == 0:
        ValueError('Argument stats2 is empty list!')

    # Check for matching list lengths
    if len(stats1) != len(stats2):
        ValueError('Arguments stats1 and stats2 have different list lengths.\n' +
                   'len(stats1) = ' + str(len(stats1)) + ' != len(stats2) = ' +
                   str(len(stats2)))
    
    # Search for duplicate pairs of statistics
    duplicates = []
    n = len(stats1)
    for i in range(n):
        for j in range(i+1,n):
            diff1 = abs((stats1[i] - stats1[j])/stats1[i])
            diff2 = abs((stats2[i] - stats2[j])/stats2[i])
            if diff1 < threshold and diff2 < threshold:
                duplicates.append( (i, j, (stats1[i], stats2[i]), (stats1[j], stats2[j])) )
 
    return duplicates




import numpy as np

def brier_score(forecast,observed):
    '''
    Calculate Brier score (BS) between two variables

    Calculates the Brier score (BS), a measure of the mean-square error of 
    probability forecasts for a dichotomous (two-category) event, such as 
    the occurrence/non-occurrence of precipitation. The score is calculated
    using the formula:

    BS = sum_(n=1)^N (f_n - o_n)^2/N
 
    where f is the forecast probabilities, o is the observed probabilities 
    (0 or 1), and N is the total number of values in f & o. Note that f & o 
    must have the same number of values, and those values must be in the 
    range [0,1].
 
    Input:
    FORECAST : forecast probabilities
    OBSERVED : observed probabilities
 
    Output:
    BS : Brier score
 
    Reference:
    Glenn W. Brier, 1950: Verification of forecasts expressed in terms 
    of probabilities. Mon. We. Rev., 78, 1-23.

    D. S. Wilks, 1995: Statistical Methods in the Atmospheric Sciences. 
    Cambridge Press. 547 pp.

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 15, 2016
    '''

    # Check that dimensions of forecast and observed fields match
    fdims= forecast.shape
    odims= observed.shape
    if not np.array_equal(fdims,odims):
        message = 'forecast and observed field dimensions do not' + \
            ' match.\n' + \
            'shape(forecast)= ' + str(fdims) + ', ' + \
            'shape(observed)= ' + str(odims) + \
            '\nforecast type: ' + str(type(forecast))
        raise ValueError(message)

    # Check for valid values
    index = np.where(np.logical_or(forecast < 0, forecast > 1))
    if sum(index) > 0:
        raise ValueError('Forecast has values outside interval [0,1].')
    index = np.where(np.logical_and(observed != 0, observed != 1))
    if sum(index) > 0:
        raise ValueError('Observed has values not equal to 0 or 1.')

    # Calculate score
    bs = np.sum(np.square(forecast - observed))/len(forecast)

    return bs



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def date_offset(years=0,
                months=0,
                weeks=0,
                days=0,
                hours=0,
                minutes=0,
                seconds=0,
                microseconds=0,
                columns=None,
                dropna='no',
                clean=False,
                skiprows=None,
                index_type='datetime',
                names=None,
                input_ts='-',
                start_date=None,
                end_date=None,
                round_index=None):
    """Apply an offset to a time-series.

    Parameters
    ----------
    years: number
        [optional, default is 0]

        Relative number of years to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    months: number
        [optional, default is 0]

        Relative number of months to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    weeks: number
        [optional, default is 0]

        Relative number of weeks to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    days: number
        [optional, default is 0]

        Relative number of days to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    hours: number
        [optional, default is 0]

        Relative number of hours to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    minutes: number
        [optional, default is 0]

        Relative number of minutes to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    seconds: number
        [optional, default is 0]

        Relative number of seconds to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
    microseconds: number
        [optional, default is 0]

        Relative number of microseconds to offset the datetime index,
        may be negative; adding or subtracting a relativedelta with
        relative information performs the corresponding arithmetic
        operation on the original datetime value with the information in
        the relativedelta.
    {input_ts}
    {start_date}
    {end_date}
    {columns}
    {round_index}
    {dropna}
    {clean}
    {skiprows}
    {index_type}
    {names}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna='no',
                              clean=clean)

    relativedelta = pd.tseries.offsets.relativedelta
    ntsd = pd.DataFrame(tsd.values,
                        index=[i +
                               relativedelta(years=years,
                                             months=months,
                                             days=days,
                                             hours=hours,
                                             minutes=minutes,
                                             seconds=seconds,
                                             microseconds=microseconds)
                               for i in tsd.index])
    ntsd.columns = tsd.columns

    return tsutils.printiso(ntsd, showindex='always')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def converttz(fromtz,
              totz,
              input_ts='-',
              columns=None,
              start_date=None,
              end_date=None,
              round_index=None,
              dropna='no',
              clean=False,
              index_type='datetime',
              names=None,
              skiprows=None):
    """Convert the time zone of the index.

    Parameters
    ----------
    fromtz : str
        The time zone of the original time-series.
    totz : str
        The time zone of the converted time-series.
    {input_ts}
    {start_date}
    {end_date}
    {columns}
    {dropna}
    {clean}
    {skiprows}
    {index_type}
    {names}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    tsd = tsd.tz_localize(fromtz).tz_convert(totz)
    return tsutils.printiso(tsd,
                            showindex='always')



import numpy as np

def bias(predicted,reference):
    '''
    Calculate the bias (B) between two variables PREDICTED and 
    REFERENCE (E'). The latter is calculated using the formula:

    B = mean(p) - mean(r)

    where p is the predicted values, and r is the reference values.
    Note that p & r must have the same number of values.

    Input:
    PREDICTED : predicted field
    REFERENCE : reference field

    Output:
    B : bias between predicted and reference

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 9, 2016
    '''

    # Check that dimensions of predicted and reference fields match
    pdims= predicted.shape
    rdims= reference.shape
    if not np.array_equal(pdims,rdims):
        message = 'predicted and reference field dimensions do not' + \
            ' match.\n' + \
            'shape(predicted)= ' + str(pdims) + ', ' + \
            'shape(reference)= ' + str(rdims) + \
            '\npredicted type: ' + str(type(predicted))
        raise ValueError(message)

    # Calculate means
    b = np.mean(predicted) - np.mean(reference)

    return b



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def rolling_window(window=2,
                   input_ts='-',
                   columns=None,
                   start_date=None,
                   end_date=None,
                   dropna='no',
                   skiprows=None,
                   index_type='datetime',
                   names=None,
                   clean=False,
                   span=None,
                   statistic='',
                   min_periods=None,
                   center=False,
                   win_type=None,
                   on=None,
                   closed=None,
                   print_input=False):
    """Calculate a rolling window statistic.

    Parameters
    ----------
    window : int, or offset
        [optional, default = 2]

        Size of the moving window. This is the number of observations used for
        calculating the statistic. Each window will be a fixed size.

        If its an offset then this will be the time period of each window. Each
        window will be a variable sized based on the observations included in
        the time-period. This is only valid for datetimelike indexes.

    min_periods : int, default None

        Minimum number of observations in window required to have a value
        (otherwise result is NA). For a window that is specified by an offset,
        this will default to 1.

    center : boolean, default False

        Set the labels at the center of the window.

    win_type : string, default None

        Provide a window type. If None, all points are evenly weighted. See the
        notes below for further information.

    on : string, optional

        For a DataFrame, column on which to calculate the rolling window,
        rather than the index

    closed : string, default None

        Make the interval closed on the 'right', 'left', 'both' or 'neither'
        endpoints. For offset-based windows, it defaults to 'right'. For fixed
        windows, defaults to 'both'. Remaining cases not implemented for fixed
        windows.

    span : int
        [optional, default = 2]

        DEPRECATED: Changed to 'window' to be consistent with pandas.

    statistic : str
        [optional, default is '']

        +----------+--------------------+
        | corr     | correlation        |
        +----------+--------------------+
        | count    | count of numbers   |
        +----------+--------------------+
        | cov      | covariance         |
        +----------+--------------------+
        | kurt     | kurtosis           |
        +----------+--------------------+
        | max      | maximum            |
        +----------+--------------------+
        | mean     | mean               |
        +----------+--------------------+
        | median   | median             |
        +----------+--------------------+
        | min      | minimum            |
        +----------+--------------------+
        | quantile | quantile           |
        +----------+--------------------+
        | skew     | skew               |
        +----------+--------------------+
        | std      | standard deviation |
        +----------+--------------------+
        | sum      | sum                |
        +----------+--------------------+
        | var      | variance           |
        +----------+--------------------+

    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {print_input}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              dropna=dropna,
                              clean=clean)

    if span is not None:
        window = span

    ntsd = tsd.rolling(window,
                       min_periods=min_periods,
                       center=center,
                       win_type=win_type,
                       on=on,
                       closed=closed)

    if statistic:
        ntsd = eval('ntsd.{0}()'.format(statistic))

    return tsutils.print_input(print_input,
                               tsd,
                               ntsd,
                               '_rolling_{0}_{1}'.format(window, statistic))



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def remove_trend(input_ts='-',
                 columns=None,
                 start_date=None,
                 end_date=None,
                 dropna='no',
                 skiprows=None,
                 index_type='datetime',
                 names=None,
                 clean=False,
                 round_index=None,
                 print_input=False):
    """Remove a 'trend'.

    Parameters
    ----------
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}
    {print_input}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    ntsd = tsd.copy()
    for col in tsd.columns:
        index = tsd.index.astype('l')
        index = index - index[0]
        lin = pd.np.polyfit(index, tsd[col], 1)
        ntsd[col] = lin[0] * index + lin[1]
        ntsd[col] = tsd[col] - ntsd[col]
    return tsutils.print_input(
        print_input, tsd, ntsd, '_rem_trend')



import matplotlib.pyplot as plt
import numpy as np

def plot_taylor_obs(ax,obsSTD, axes, option):
    '''
    Plots observation STD on Taylor diagram.
    
    Optionally plots a marker on the x-axis indicating observation STD, 
    a label for this point, and a contour circle indicating the STD 
    value.
    
    INPUTS:
    ax     : axes handle for Taylor diagram
    obsSTD : observation standard deviation
    axes   : data structure containing axes information for target diagram
    option : data structure containing option values. (Refer to 
             GET_TAYLOR_DIAGRAM_OPTIONS function for more information.)
    option['colobs']       : color for observation labels (Default : magenta)
    option['markerobs']    : marker to use for x-axis indicating observed STD
    option['styleobs']     : line style for observation grid line
    option['titleobs']     : label for observation point label (Default: '')
    option['widthobs']     : linewidth for observation grid line (Default: .8)
 
    OUTPUTS:
    None
    
    Author: Peter A. Rochford
            Symplectic, LLC
            www.thesymplectic.com
            prochford@thesymplectic.com

    Created on Feb 19, 2017

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    '''
    
    if option['markerobs'] != 'none':
        # Display marker on x-axis indicating observed STD
        yobsSTD = 0.001*axes['rmax'] -axes['rmin']
        plt.plot(obsSTD,yobsSTD,option['markerobs'],color = option['colobs'],
                 markersize = 6, markerfacecolor = option['colobs'], 
                 markeredgecolor = option['colobs'],
                 linewidth = 1.0, clip_on=False);
    
    if option['titleobs'] != '':
        # Put label below the marker
        labelweight = 'bold'
        labelsize = ax[0].get_fontsize() - 2
        x = obsSTD; y = -0.05*axes['rmax'];
        plt.text(x,y,option['titleobs'], color = option['colobs'],
                 horizontalalignment = 'center', 
                 fontweight = labelweight, fontsize = labelsize)

    
    if option['styleobs'] != '':
        # Draw circle for observation STD
        theta = np.arange(0, 2*np.pi, np.pi/150)
        xunit = obsSTD*np.cos(theta)
        yunit = obsSTD*np.sin(theta)
        plt.plot(xunit,yunit,linestyle=option['styleobs'],
                 color = option['colobs'],linewidth = option['widthobs'])



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def createts(freq=None,
             fillvalue=None,
             input_ts=None,
             index_type='datetime',
             start_date=None,
             end_date=None):
    """Create empty time series, optionally fill with a value.

    Parameters
    ----------
    freq : str
        [optional, default is None]

        To use this form `--start_date` and `--end_date` must be supplied
        also.  The `freq` option is the pandas date offset code used to create
        the index.
    fillvalue
        [optional, default is None]

        The fill value for the time-series.  The default is None, which
        generates the date/time stamps only.
    {input_ts}
    {start_date}
    {end_date}
    {index_type}

    """
    if input_ts is None:
        assert ((start_date is not None) and
                (end_date is not None) and
                (freq is not None)), """
*
*   If input_ts is None, then start_date, end_date, and freq must be supplied.
*   Instead you have:
*   start_date = {0},
*   end_date = {1},
*   freq = {2}
*
""".format(start_date, end_date, freq)

    if input_ts is not None:
        tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                      index_type=index_type),
                                  start_date=start_date,
                                  end_date=end_date)
        tsd = pd.DataFrame([fillvalue] * len(tsd.index),
                           index=tsd.index)
    else:
        tindex = pd.date_range(start=start_date,
                               end=end_date,
                               freq=freq)
        tsd = pd.DataFrame([fillvalue] * len(tindex),
                           index=tindex)
    return tsutils.printiso(tsd,
                            showindex='always')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def calculate_fdc(input_ts='-',
                  columns=None,
                  start_date=None,
                  end_date=None,
                  clean=False,
                  skiprows=None,
                  index_type='datetime',
                  names=None,
                  percent_point_function=None,
                  plotting_position='weibull',
                  ascending=True):
    """Return the frequency distribution curve.

    DOES NOT return a time-series.

    Parameters
    ----------
    percent_point_function : str
        [optional, default is None]

        The distribution used to shift the plotting position values.
        Choose from 'norm', 'lognorm', 'weibull', and None.
    plotting_position : str
        [optional, default is 'weibull']

        {plotting_position_table}

    ascending : bool
        Sort order defaults to True.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {skiprows}
    {index_type}
    {names}
    {clean}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              clean=clean)

    ppf = tsutils._set_ppf(percent_point_function)
    newts = pd.DataFrame()
    for col in tsd:
        tmptsd = tsd[col].dropna()
        xdat = ppf(tsutils._set_plotting_position(tmptsd.count(),
                                                  plotting_position)) * 100
        tmptsd.sort_values(ascending=ascending, inplace=True)
        tmptsd.index = xdat
        newts = newts.join(tmptsd, how='outer')
    newts.index.name = 'Plotting_position'
    newts = newts.groupby(newts.index).first()
    return tsutils.printiso(newts,
                            showindex='always')



import matplotlib.ticker as ticker
import matplotlib.pyplot as plt
import numpy as np

def get_taylor_diagram_axes(rho,option):
    '''
    Get axes value for taylor_diagram function.
    
    Determines the axes information for a Taylor diagram given the axis 
    values (X,Y) and the options in the dictionary OPTION returned by 
    the GET_TAYLOR_DIAGRAM_OPTIONS function.
 
    INPUTS:
    rho    : radial coordinate
    option : dictionary containing option values. (Refer to 
             GET_TAYLOR_DIAGRAM_OPTIONS function for more information.)
 
    OUTPUTS:
    axes         : dictionary containing axes information for Taylor diagram
    axes['dx']   : observed standard deviation
    axes['next'] : directive on how to add next plot
    axes['rinc'] : increment for radial coordinate
    axes['rmax'] : maximum value for radial coordinate
    axes['rmin'] : minimum value for radial coordinate
    axes['tc']   : color for x-axis
    option       : dictionary containing updated option values
  
    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 3, 2016
    '''

    axes = {}
    axes['dx'] = rho[0]
       
    cax = plt.gca()
    axes['tc'] = cax.xaxis.label.get_color()
    axes['next'] = 'replace' #needed?
    
    # make a radial grid
    if option['axismax'] == 0.0:
        maxrho = max(abs(rho))
    else:
        maxrho = option['axismax'];

    # Determine default number of tick marks
    yticks = ticker.AutoLocator().tick_values(-1.0*maxrho, maxrho)
    ticks = np.sum(yticks >= 0)
    
    # Check radial limits and ticks
    axes['rmin'] = 0; 
    if option['axismax'] == 0.0:
        axes['rmax'] = yticks[-1]
        option['axismax'] = axes['rmax']
    else:
        axes['rmax'] = option['axismax']
    rticks = np.amax(ticks-1,axis=0)
    if rticks > 5: # see if we can reduce the number
        if rticks % 2 == 0:
            rticks = rticks/2
        elif rticks % 3 == 0:
            rticks = rticks/3
    axes['rinc']  = (axes['rmax'] - axes['rmin'])/rticks
    tick  = np.arange(axes['rmin'] + axes['rinc'],axes['rmax'] + axes['rinc'],axes['rinc']) 
    
    if len(option['tickrms']) == 0:
        option['tickrms'] = tick; option['rincrms'] = axes['rinc']
    if len(option['tickstd']) == 0:
        option['tickstd'] = tick; option['rincstd'] = axes['rinc']
    
    return axes, cax



import matplotlib.pyplot as plt
import matplotlib.colors as clr
import warnings

def plot_pattern_diagram_markers(X,Y,option):
    '''
    Plots color markers on a pattern diagram.
    
    Plots color markers on a target diagram according their (X,Y) 
    locations. The symbols and colors are chosen automatically with a 
    limit of 70 symbol & color combinations.
    
    The color bar is titled using the content of option['titleColorBar'] 
    (if non-empty string).
    
    INPUTS:
    x : x-coordinates of markers
    y : y-coordinates of markers
    z : z-coordinates of markers (used for color shading)
    option : dictionary containing option values. (Refer to 
        GET_TARGET_DIAGRAM_OPTIONS function for more information.)
    option['axismax'] : maximum for the X & Y values. Used to limit
        maximum distance from origin to display markers
    option['markerlabel'] : labels for markers
    
    OUTPUTS:
    None

    Created on Nov 30, 2016
    
    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    '''

    # Set face color transparency
    alpha = option['alpha']
    
    # Set marker size
    markerSize = option['markersize']
    
    if option['markerlegend'] == 'on':
        # Check that marker labels have been provided
        if option['markerlabel'] == '':
            raise ValueError('No marker labels provided.')

        # Plot markers of different color and shapes with labels 
        # displayed in a legend
        
        # Define markers
        kind = ['+','o','x','s','d','^','v','p','h','*']
        colorm = ['b','r','g','c','m','y','k']
        if len(X) > 70:
            _disp('You must introduce new markers to plot more than 70 cases.')
            _disp('The ''marker'' character array need to be extended inside the code.')
        
        if len(X) <= len(kind):
            # Define markers with specified color
            marker = []
            markercolor = []
            for color in colorm:
                for symbol in kind:
                    marker.append(symbol + option['markercolor'])
                    rgba = clr.to_rgb(option['markercolor']) + (alpha,)
                    markercolor.append(rgba)
        else:
            # Define markers and colors using predefined list
            marker = []
            for color in colorm:
                for symbol in kind:
                    marker.append(symbol + color)
                    rgba = clr.to_rgb(color) + (alpha,)
                    markercolor.append(rgba)
        
        # Plot markers at data points
        limit = option['axismax']
        hp = ()
        markerlabel = []
        for i, xval in enumerate(X):
            if abs(X[i]) <= limit and abs(Y[i]) <= limit:
                h = plt.plot(X[i],Y[i],marker[i], markersize = markerSize, 
                     markerfacecolor = markercolor[i],
                     markeredgecolor = marker[i][1],
                     markeredgewidth = 2)
                hp += tuple(h)
                markerlabel.append(option['markerlabel'][i])
        
        # Add legend
        if len(markerlabel) > 0:
            markerlabel = tuple(markerlabel)
            plt.legend(hp, markerlabel, loc = 'upper right', 
                       fontsize = 'medium', numpoints=1)
        else:
            warnings.warn('No markers within axis limit ranges.')

    else:
        # Plot markers as dots of a single color with accompanying labels
        # and no legend
        
        # Plot markers at data points
        limit = option['axismax']
        rgba = clr.to_rgb(option['markercolor']) + (alpha,) 
        for i,xval in enumerate(X):
            if abs(X[i]) <= limit and abs(Y[i]) <= limit:
                # Plot marker
                plt.plot(X[i],Y[i],'.', markersize = markerSize, 
                     markerfacecolor = rgba,
                     markeredgecolor = option['markercolor'])
                
                # Check if marker labels provided
                if len(option['markerlabel']) > 0:
                    # Label marker
                    xtextpos = X[i] #ToDo: convert to double?
                    ytextpos = Y[i]
                    plt.text(xtextpos,ytextpos,option['markerlabel'][i], 
                             color = option['markerlabelcolor'],
                             verticalalignment = 'bottom',
                             horizontalalignment = 'right',
                             fontsize = 'medium')

def _disp(text):
    print(text)



import numpy as np

def centered_rms_dev(predicted,reference):
    '''
    Calculates the centered root-mean-square (RMS) difference between 
    two variables PREDICTED and REFERENCE (E'). The latter is calculated 
    using the formula:

    (E')^2 = sum_(n=1)^N [(p_n - mean(p))(r_n - mean(r))]^2/N

    where p is the predicted values, r is the reference values, and
    N is the total number of values in p & r. Note that p & r must
    have the same number of values.

    Input:
    PREDICTED : predicted field
    REFERENCE : reference field

    Output:
    CRMSDIFF : centered root-mean-square (RMS) difference (E')^2

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Nov 24, 2016
    '''

    # Check that dimensions of predicted and reference fields match
    pdims= predicted.shape
    rdims= reference.shape
    if not np.array_equal(pdims,rdims):
        message = 'predicted and reference field dimensions do not' + \
            ' match.\n' + \
            'shape(predicted)= ' + str(pdims) + ', ' + \
            'shape(reference)= ' + str(rdims) + \
            '\npredicted type: ' + str(type(predicted))
        raise ValueError(message)

    # Calculate means
    pmean = np.mean(predicted)
    rmean = np.mean(reference)

    # Calculate (E')^2
    crmsd = np.square((predicted - pmean) - (reference - rmean))
    crmsd = np.sum(crmsd)/predicted.size
    crmsd = np.sqrt(crmsd)

    return crmsd



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def normalization(input_ts='-',
                  columns=None,
                  start_date=None,
                  end_date=None,
                  dropna='no',
                  skiprows=None,
                  index_type='datetime',
                  names=None,
                  clean=False,
                  mode='minmax',
                  min_limit=0,
                  max_limit=1,
                  pct_rank_method='average',
                  print_input=False,
                  round_index=None,
                  float_format='%g'):
    """Return the normalization of the time series.

    Parameters
    ----------
    mode : str
        [optional, default is 'minmax']

        minmax
            min_limit +
            (X-Xmin)/(Xmax-Xmin)*(max_limit-min_limit)

        zscore
            X-mean(X)/stddev(X)

        pct_rank
            rank(X)*100/N
    min_limit : float
        [optional, defaults to 0]

        Defines the minimum limit of the minmax normalization.
    max_limit : float
        [optional, defaults to 1]

        Defines the maximum limit of the minmax normalization.
    pct_rank_method : str
        [optional, defaults to 'average']

        Defines how tied ranks are broken.  Can be 'average', 'min', 'max',
        'first', 'dense'.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {print_input}
    {float_format}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)

    # Trying to save some memory
    if print_input:
        otsd = tsd.copy()
    else:
        otsd = pd.DataFrame()

    if mode == 'minmax':
        tsd = (min_limit +
               (tsd - tsd.min()) /
               (tsd.max() - tsd.min()) *
               (max_limit - min_limit))
    elif mode == 'zscore':
        tsd = (tsd - tsd.mean()) / tsd.std()
    elif mode == 'pct_rank':
        tsd = tsd.rank(method=pct_rank_method, pct=True)
    else:
        raise ValueError("""
*
*   The 'mode' options are 'minmax', 'zscore', or 'pct_rank', you gave me
*   {0}.
*
""".format(mode))

    tsd = tsutils.memory_optimize(tsd)
    return tsutils.print_input(print_input,
                               otsd,
                               tsd,
                               '_{0}'.format(mode),
                               float_format=float_format)



import matplotlib.pyplot as plt
import numpy as np
from numpy import where

def overlay_taylor_diagram_circles(axes,cax,option):
    '''
    Overlays circle contours on a Taylor diagram.
    
    Plots circle contours on a Taylor diagram to indicate root mean square 
    (RMS) and standard deviation values.
    
    INPUTS:
    axes   : data structure containing axes information for Taylor diagram
    cax    : handle for plot axes
    option : data structure containing option values. (See 
             GET_TAYLOR_DIAGRAM_OPTIONS for more information.)
    option['colrms']       : RMS grid and tick labels color (Default: green)
    option['rincrms']      : Increment spacing for RMS grid
    option['stylerms']     : Linestyle of the RMS grid
    option['tickrms']      : RMS values to plot gridding circles from 
                             observation point
    option['tickRMSangle'] : Angle for RMS tick labels with the observation 
                             point (Default: 135 deg.)
    option['wi%    dthrms']     : Line width of the RMS grid
 
    option['colstd']       : STD grid and tick labels color (Default: black)
    option['rincstd']      : Increment spacing for STD grid
    option['stylestd']     : Linestyle of the STD grid
    option['tickstd']      : STD values to plot gridding circles from origin
    option['tickstdangle'] : Angle for STD tick labels with the observation 
                             point (Default: .8)
    option['widthstd']     : Line width of the STD grid
 
    OUTPUTS:
    None.
 
    See also GET_TAYLOR_DIAGRAM_OPTIONS

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    '''

    th = np.arange(0, 2*np.pi, np.pi/150)
    xunit = np.cos(th)
    yunit = np.sin(th)

    # now really force points on x/y axes to lie on them exactly
    inds = range(0,len(th),(len(th)-1) // 4)
    xunit[inds[1:5:2]] = np.zeros(2)
    yunit[inds[0:6:2]] = np.zeros(3)
    
    # DRAW RMS CIRCLES:
    # ANGLE OF THE TICK LABELS
    if option['tickrmsangle'] > 0:
        tickRMSAngle = option['tickrmsangle']
    else:
        phi = np.arctan2(option['tickstd'][-1],axes['dx'])
        tickRMSAngle = 180 - np.rad2deg(phi)
    
    c82 = np.cos(tickRMSAngle*np.pi/180)
    s82 = np.sin(tickRMSAngle*np.pi/180)
    radius = np.sqrt(axes['dx']**2+axes['rmax']**2 - 
                     2*axes['dx']*axes['rmax']*xunit)

    # Define label format
    labelFormat = '{' + option['rmslabelformat'] + '}'
    
    for iradius in option['tickrms']:
        phi = th[np.where(radius >= iradius)]
        phi = phi[0]
        ig = np.where(iradius*np.cos(th)+axes['dx'] <= 
                      axes['rmax']*np.cos(phi))
        hhh = plt.plot(xunit[ig]*iradius+axes['dx'],yunit[ig]*iradius, 
                       linestyle = option['stylerms'],color = option['colrms'], 
                       linewidth = option['widthrms'])
        if option['showlabelsrms'] == 'on':
            xtextpos = (iradius+option['rincrms']/20)*c82 + axes['dx']
            ytextpos = (iradius+option['rincrms']/20)*s82
            plt.text(xtextpos,ytextpos, '  ' + labelFormat.format(iradius), 
                     verticalalignment = 'baseline', 
                     color = option['colrms'], rotation = tickRMSAngle - 90)
    
    # DRAW STD CIRCLES:
    # draw radial circles
    for i in option['tickstd']:
        hhh = plt.plot(xunit*i,yunit*i,linestyle = option['stylestd'],
                   color = option['colstd'], linewidth = option['widthstd'])
        if option['showlabelsstd'] == 'on':
            if option['numberpanels'] == 2:
                if len(where(option['tickstd']==0)) == 0:
                    plt.text(0,-axes['rinc']/20,'0', verticalalignment = 'top',
                         horizontalalignment = 'center', color = option['colstd'])
                plt.text(i,-axes['rinc']/20,str(i), verticalalignment = 'top',
                     horizontalalignment = 'center', color = option['colstd'])
                plt.text(-i,-axes['rinc']/20,str(i), verticalalignment = 'top',
                     horizontalalignment = 'center', color = option['colstd'])
            else:
                if len(where(option['tickstd']==0)) == 0:
                    plt.text(-axes['rinc']/20,axes['rinc']/20,'0', 
                        verticalalignment = 'center', 
                        horizontalalignment = 'right',
                        color = option['colstd'])
                plt.text(-axes['rinc']/20,i, str(i), 
                        verticalalignment = 'center',
                        horizontalalignment = 'right',
                        color = option['colstd'])

    hhh[0].set_linestyle('-') # Make outermost STD circle solid
    
    # Draw circle for outer boundary
    i = option['axismax']
    hhh = plt.plot(xunit*i,yunit*i,linestyle = option['stylestd'],
               color = option['colstd'], linewidth = option['widthstd'])



import matplotlib.pyplot as plt
import numpy as np

def overlay_taylor_diagram_lines(axes,cax,option):
    '''
    Overlay lines emanating from origin on a Taylor diagram.

    OVERLAY_TAYLOR_DIAGRAM_CIRCLES(AXES,CAX,OPTION)
    Plots lines emanating from origin to indicate correlation values (CORs) 
 
    INPUTS:
    axes   : data structure containing axes information for target diagram
    cax    : handle for plot axes
    option : data structure containing option values. (Refer to 
             GET_TAYLOR_DIAGRAM_OPTIONS function for more information.)
    option['colcor']   : CORs grid and tick labels color (Default: blue)
    option['showlabelscor'] : Show or not the CORRELATION tick labels
    option['stylecor'] : Linestyle of the CORs grid
    option['tickcor']  : CORs values to plot lines from origin
    option['widthcor'] : Line width of the CORs grid
 
    OUTPUTS:
    None.
    
    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 3, 2016
   '''

    # DRAW CORRELATION LINES EMANATING FROM THE ORIGIN:
    npanels = option['numberpanels']
    corr = option['tickcor'][npanels-1]
    th  = np.arccos(corr)
    cst = np.cos(th); snt = np.sin(th);
    cs = np.append(-1.0*cst, cst)
    sn = np.append(-1.0*snt, snt)
    for i,val in enumerate(cs):
        plt.plot([0, axes['rmax']*cs[i]],[0, axes['rmax']*sn[i]], 
                 linestyle = option['stylecor'],
                 color = option['colcor'], linewidth = option['widthcor'])
    
    # annotate the lines by correlation coefficient
    if option['showlabelscor'] == 'on':
        rt = 1.05*axes['rmax']
        for i,cc in enumerate(corr):
            plt.text(rt*cst[i],rt*snt[i],str(round(cc,2)), 
                     horizontalalignment = 'center', color = option['colcor'])



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import numpy as np
import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


class MisMatchedKernel(Exception):
    """Error class for the wrong length kernel."""

    def __init__(self, rk, pw):
        """Initialize with lengths of kernel and requested length."""
        self.rk = rk
        self.pw = pw

    def __str__(self):
        """Return detailed error message."""
        return """
*
*   Length of kernel must be {0}.
*   Instead have {1}
*
""".format(self.rk, self.pw)


class BadKernelValues(Exception):
    """Error class for the negative pad width."""

    def __str__(self):
        """Return detailed error message."""
        return """
*
*   Should only have positive values.
*
"""


def _transform(vector, cutoff_period, window_len, lopass=None):
    """Private function used by FFT filtering.

    Parameters
    ----------
    vector : array_like, evenly spaced samples in time

    Returns
    -------
    vector of filtered values

    """
    if cutoff_period is None:
        raise ValueError("""
*
*   The cutoff_period must be set.
*
""")

    if window_len is None:
        raise ValueError("""
*
*   The window_len must be set.
*
""")

    import numpy.fft as F
    result = F.rfft(vector, len(vector))

    freq = F.fftfreq(len(vector))[:len(vector)//2 + 1]
    factor = np.ones_like(freq)

    if lopass is True:
        factor[freq > 1.0/float(cutoff_period)] = 0.0
        factor = np.pad(factor, window_len + 1, mode='constant',
                        constant_values=(1.0, 0.0))
    else:
        factor[freq < 1.0/float(cutoff_period)] = 0.0
        factor = np.pad(factor, window_len + 1, mode='constant',
                        constant_values=(0.0, 1.0))

    factor = np.convolve(factor, [1.0/window_len]*window_len, mode=1)
    factor = factor[window_len + 1:-(window_len + 1)]

    result = result * factor

    rvector = F.irfft(result, len(vector))

    return np.atleast_1d(rvector)


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def filter(filter_type,
           input_ts='-',
           columns=None,
           start_date=None,
           end_date=None,
           dropna='no',
           skiprows=None,
           index_type='datetime',
           names=None,
           clean=False,
           print_input=False,
           cutoff_period=None,
           window_len=5,
           float_format='%g',
           round_index=None):
    """Apply different filters to the time-series.

    Parameters
    ----------
    filter_type : str
        'flat', 'hanning', 'hamming', 'bartlett', 'blackman',
        'fft_highpass' and 'fft_lowpass' for Fast Fourier Transform
        filter in the frequency domain.
    window_len : int
        [optional, default is 5]

        For the windowed types, 'flat', 'hanning', 'hamming',
        'bartlett', and 'blackman' specifies the length of the window.
    cutoff_period
        [optional, default is None]

        For 'fft_highpass' and 'fft_lowpass'.  Must be supplied if using
        'fft_highpass' or 'fft_lowpass'.  The period in input time units that
        will form the cutoff between low frequencies (longer periods) and high
        frequencies (shorter periods).  Filter will be smoothed by `window_len`
        running average.
    {input_ts}
    {start_date}
    {end_date}
    {columns}
    {float_format}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}
    {print_input}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)

    assert len(tsd.values) > window_len, """
*
*   Input vector (length={0}) needs to be bigger than window size ({1}).
*
""".format(len(tsd.values), window_len)

    assert filter_type in ['flat',
                           'hanning',
                           'hamming',
                           'bartlett',
                           'blackman',
                           'fft_highpass',
                           'fft_lowpass'], """
*
*   Filter type {0} not implemented.
*
""".format(filter_type)

    # Trying to save some memory
    if print_input:
        otsd = tsd.copy()
    else:
        otsd = pd.DataFrame()

    for col in tsd.columns:
        # fft_lowpass, fft_highpass
        if filter_type == 'fft_lowpass':
            tsd[col].values[:] = _transform(tsd[col].values,
                                            cutoff_period,
                                            window_len,
                                            lopass=True)
        elif filter_type == 'fft_highpass':
            tsd[col].values[:] = _transform(tsd[col].values,
                                            cutoff_period,
                                            window_len)
        elif filter_type in ['flat',
                             'hanning',
                             'hamming',
                             'bartlett',
                             'blackman']:
            if window_len < 3:
                continue
            s = pd.np.pad(tsd[col].values, window_len // 2, 'reflect')

            if filter_type == 'flat':  # moving average
                w = pd.np.ones(window_len, 'd')
            else:
                w = eval('pd.np.' + filter_type + '(window_len)')
            tsd[col].values[:] = pd.np.convolve(w / w.sum(), s, mode='valid')

    return tsutils.print_input(print_input, otsd, tsd, '_filter',
                               float_format=float_format)



import numpy as np
from . import rmsd

def skill_score_murphy(predicted,reference):
    '''
    Calculate non-dimensional skill score (SS) between two variables using
    definition of Murphy (1988)

    Calculates the non-dimensional skill score (SS) difference between two 
    variables PREDICTED and REFERENCE. The skill score is calculated using 
    the formula:
 
    SS = 1 - RMSE^2/SDEV^2
 
    where RMSE is the root-mean-squre error between the predicted and
    reference values
 
    (RMSE)^2 = sum_(n=1)^N (p_n - r_n)^2/N
 
    and SDEV is the standard deviation of the reference values
 
    SDEV^2 = sum_(n=1)^N [r_n - mean(r)]^2/(N-1)
 
    where p is the predicted values, r is the reference values, and
    N is the total number of values in p & r. Note that p & r must
    have the same number of values.
 
    Input:
    PREDICTED : predicted field
    REFERENCE : reference field
 
    Output:
    SS : skill score

    Reference:
    Allan H. Murphy, 1988: Skill Scores Based on the Mean Square Error 
    and Their Relationships to the Correlation Coefficient. Mon. Wea. 
    Rev., 116, 2417-2424.
    doi: http//dx.doi.org/10.1175/1520-0493(1988)<2417:SSBOTM>2.0.CO;2
    
    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 7, 2016
    '''

    # Check that dimensions of predicted and reference fields match
    pdims= predicted.shape
    rdims= reference.shape
    if not np.array_equal(pdims,rdims):
        message = 'predicted and reference field dimensions do not' + \
            ' match.\n' + \
            'shape(predicted)= ' + str(pdims) + ', ' + \
            'shape(reference)= ' + str(rdims) + \
            '\npredicted type: ' + str(type(predicted))
        raise ValueError(message)
    
    # Calculate RMSE
    rmse2 = rmsd(predicted,reference)**2
    
    # Calculate standard deviation
    sdev2 = np.std(reference,ddof=1)**2

    #% Calculate skill score
    ss = 1 - rmse2/sdev2

    return ss



import numpy as np

def check_taylor_stats(STDs, CRMSDs, CORs, threshold = 0.01):
    '''
    Checks input statistics satisfy Taylor diagram relation to <1%.

    Function terminates with an error if not satisfied. The threshold is
    the ratio of the difference between the statistical metrics and the
    centered root mean square difference:

     abs(CRMSDs^2 - (STDs^2 + STDs(1)^2 - 2*STDs*STDs(1)*CORs))/CRMSDs^2

    Note that the first element of the statistics vectors must contain
    the value for the reference field.

    INPUTS:
    STDs      : Standard deviations
    CRMSDs    : Centered Root Mean Square Difference 
    CORs      : Correlation
    threshold : limit for acceptance, e.g. 0.1 for 10% (default 0.01)

    OUTPUTS:
    None.

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 3, 2016
    '''
    if threshold < 1e-7:
        ValueError('threshold value must be positive: ' + str(threshold))

    diff = np.square(CRMSDs[1:]) \
           - (np.square(STDs[1:]) + np.square(STDs[0]) \
           - 2.0*STDs[0]*np.multiply(STDs[1:],CORs[1:]))
    diff = np.abs(np.divide(diff,np.square(CRMSDs[1:])))
    index = np.where(diff > threshold)
    if len(index) > 0:
        ii = np.where(diff != 0)
        if len(ii) == len(diff):
            ValueError('Incompatible data\nYou must have:' +
                '\nCRMSDs - sqrt(STDs.^2 + STDs(1)^2 - ' +
                '2*STDs*STDs(1).*CORs) = 0 !')
        else:
            ValueError('Incompatible data indices: {}'.format(ii) +
                       '\nYou must have:\nCRMSDs - sqrt(STDs.^2 + STDs(1)^2 - ' +
                       '2*STDs*STDs(1).*CORs) = 0 !')

    return diff



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def replace(from_values,
            to_values,
            round_index=None,
            input_ts='-',
            columns=None,
            start_date=None,
            end_date=None,
            dropna='no',
            skiprows=None,
            index_type='datetime',
            names=None,
            clean=False,
            print_input=False):
    """Return a time-series replacing values with others.

    Parameters
    ----------
    from_values
        All values in this comma separated list are replaced
        with the corresponding value in to_values.  Use the
        string 'None' to represent a missing value.  If
        using 'None' as a from_value it might be easier to
        use the "fill" subcommand instead.
    to_values
        All values in this comma separater list are the
        replacement values corresponding one-to-one to the
        items in from_values.  Use the string 'None' to
        represent a missing value.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}
    {print_input}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)

    nfrom_values = []
    for fv in from_values.split(','):
        if fv == 'None':
            nfrom_values.append(None)
            continue
        try:
            nfrom_values.append(int(fv))
        except ValueError:
            nfrom_values.append(float(fv))

    nto_values = []
    for tv in to_values.split(','):
        if tv == 'None':
            nto_values.append(None)
            continue
        try:
            nto_values.append(int(tv))
        except ValueError:
            nto_values.append(float(tv))

    ntsd = tsd.replace(nfrom_values, nto_values)
    if dropna in ['any', 'all']:
        ntsd = ntsd.dropna(axis='index', how=dropna)

    return tsutils.print_input(
        print_input, tsd, ntsd, '_replace')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def date_slice(input_ts='-',
               columns=None,
               start_date=None,
               end_date=None,
               dropna='no',
               clean=False,
               skiprows=None,
               index_type='datetime',
               names=None,
               round_index=None,
               float_format='%g'):
    """Print out data to the screen between start_date and end_date.

    Parameters
    ----------
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {clean}
    {skiprows}
    {index_type}
    {names}
    {float_format}
    {round_index}

    """
    return tsutils.printiso(
        tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                skiprows=skiprows,
                                                names=names,
                                                index_type=index_type),
                            start_date=start_date,
                            end_date=end_date,
                            pick=columns,
                            round_index=round_index,
                            dropna=dropna,
                            clean=clean), float_format=float_format)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def add_trend(start_offset,
              end_offset,
              input_ts='-',
              columns=None,
              clean=False,
              start_date=None,
              end_date=None,
              dropna='no',
              round_index=None,
              skiprows=None,
              index_type='datetime',
              names=None,
              print_input=False):
    """Add a trend.

    Parameters
    ----------
    start_offset : float
        The starting value for the applied trend.
    end_offset : float
        The ending value for the applied trend.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {clean}
    {round_index}
    {skiprows}
    {index_type}
    {names}
    {print_input}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    # Need it to be float since will be using pd.np.nan
    ntsd = tsd.copy().astype('float64')

    ntsd.ix[:, :] = pd.np.nan
    ntsd.ix[0, :] = float(start_offset)
    ntsd.ix[-1, :] = float(end_offset)
    ntsd = ntsd.interpolate(method='values')

    ntsd = ntsd + tsd

    ntsd = tsutils.memory_optimize(ntsd)
    return tsutils.print_input(print_input,
                               tsd,
                               ntsd,
                               '_trend')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def stack(input_ts='-',
          columns=None,
          start_date=None,
          end_date=None,
          round_index=None,
          dropna='no',
          skiprows=None,
          index_type='datetime',
          names=None,
          clean=False):
    """Return the stack of the input table.

    The stack command takes the standard table and
    converts to a three column table.

    From::

      Datetime,TS1,TS2,TS3
      2000-01-01 00:00:00,1.2,1018.2,0.0032
      2000-01-02 00:00:00,1.8,1453.1,0.0002
      2000-01-03 00:00:00,1.9,1683.1,-0.0004

    To::

      Datetime,Columns,Values
      2000-01-01,TS1,1.2
      2000-01-02,TS1,1.8
      2000-01-03,TS1,1.9
      2000-01-01,TS2,1018.2
      2000-01-02,TS2,1453.1
      2000-01-03,TS2,1683.1
      2000-01-01,TS3,0.0032
      2000-01-02,TS3,0.0002
      2000-01-03,TS3,-0.0004

    Parameters
    ----------
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)

    newtsd = pd.DataFrame(tsd.stack()).reset_index(1)
    newtsd.columns = ['Columns', 'Values']
    newtsd = newtsd.groupby('Columns').apply(
        lambda d: d.sort_values('Columns')).reset_index('Columns', drop=True)
    return tsutils.printiso(newtsd)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def pct_change(input_ts='-',
               columns=None,
               start_date=None,
               end_date=None,
               dropna='no',
               skiprows=None,
               index_type='datetime',
               names=None,
               clean=False,
               periods=1,
               fill_method='pad',
               limit=None,
               freq=None,
               print_input=False,
               round_index=None,
               float_format='%g'):
    """Return the percent change between times.

    Parameters
    ----------
    periods : int
        [optional, default is 1]

        The number of intervals to calculate percent change across.
    fill_method : str
        [optional, defaults to 'pad']

        Fill method for NA.  Defaults to 'pad'.
    limit
        [optional, defaults to None]

        Is the minimum number of consecutive NA values where no more filling
        will be made.
    freq : str
        [optional, defaults to None]

        A pandas time offset string to represent the interval.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {print_input}
    {float_format}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)

    # Trying to save some memory
    if print_input:
        otsd = tsd.copy()
    else:
        otsd = pd.DataFrame()

    return tsutils.print_input(print_input,
                               otsd,
                               tsd.pct_change(periods=periods,
                                              fill_method=fill_method,
                                              limit=limit,
                                              freq=freq),
                               '_pct_change',
                               float_format=float_format)



import matplotlib.pyplot as plt
from matplotlib import ticker

def plot_pattern_diagram_colorbar(X,Y,Z,option):
    '''
    Plots color markers on a pattern diagram shaded according to a 
    supplied value.
    
    Values are indicated via a color bar on the plot.
    
    Plots color markers on a target diagram according their (X,Y) locations.
    The color shading is accomplished by plotting the markers as a scatter 
    plot in (X,Y) with the colors of each point specified using Z as a 
    vector.
    
    The color range is controlled by option['nonRMSDz']. 
    option['colormap'] = 'on' :
        the scatter function maps the elements in Z to colors in the 
        current colormap
    option['colormap']= 'off' : the color axis is mapped to the range
        [min(Z) max(Z)]
    
    The color bar is titled using the content of option['titleColorBar'] 
    (if non-empty string).
    
    INPUTS:
    x : x-coordinates of markers
    y : y-coordinates of markers
    z : z-coordinates of markers (used for color shading)
    option : dictionary containing option values.
    option['colormap'] : 'on'/'off' switch to map color shading of markers 
        to colormap ('on') or min to max range of Z values ('off').
    option['titleColorBar'] : title for the color bar
    
    OUTPUTS:
    None.
    
    Created on Nov 30, 2016
    
    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    '''

    '''
    Plot color shaded data points using scatter plot
    Keyword s defines marker size in points^2
            c defines the sequence of numbers to be mapped to colors 
              using the cmap and norm
    '''
    hp = plt.scatter(X,Y, s=50, c=Z, marker = 'd', edgecolors='none')
    
    # Add color bar to plot
    if option['colormap'] == 'on':
        hc = plt.colorbar(orientation='horizontal', aspect = 6,
                           fraction=0.04, pad=0.04)
        
    elif option['colormap'] == 'off':
        if len(Z) > 1:
            plt.clim(min(Z), max(Z))
            hc = plt.colorbar(orientation='horizontal', aspect = 6,
                              fraction=0.04, pad=0.04)
#                 hc.set_ticks([min(Z), max(Z)])
            hc.set_ticklabels('Min. RMSD','Max. RMSD')
    else:
        raise ValueError('Invalid option for option.colormap: ' + 
                         option['colormap']);
    
    # Set desired properties of color bar
    location = _getColorBarLocation(hc,option, xscale = 0.75, 
                                   yscale = 7.5, cxscale = 1.0)
    hc.ax.set_position(location) # set new position
    hc.ax.tick_params(labelsize='medium') #  set tick labels to medium

    # Limit number of ticks on colar bar to 4
    hc.locator = ticker.MaxNLocator(nbins=5)
    hc.update_ticks()

    hc.ax.xaxis.set_ticks_position('top')
    hc.ax.xaxis.set_label_position('top')

    # Title the color bar
    if option['titlecolorbar']:
        hc.set_label(option['titlecolorbar'])
    else:
        hc.set_label(hc,'Color Scale')

def _getColorBarLocation(hc,option,**kwargs):
    '''
    Determine location for color bar.
    
    Determines location to place color bar for type of plot:
    target diagram and Taylor diagram. Optional scale arguments
    (xscale,cxscale) can be supplied to adjust the placement of
    the colorbar to accommodate different situations.

    INPUTS:
    hc     : handle returned by colorbar function
    option : dictionary containing option values. (Refer to 
             display_target_diagram_options function for more 
             information.)
    
    OUTPUTS:
    location : x, y, width, height for color bar
    
    KEYWORDS:
    xscale  : scale factor to adjust x-position of color bar
    yscale  : scale factor to adjust y-position of color bar
    cxscale : scale factor to adjust thickness of color bar
    '''

    # Check for optional arguments and set defaults if required
    if 'xscale' in kwargs:
        xscale = kwargs['xscale']
    else:
        xscale = 1.0
    if 'yscale' in kwargs:
        yscale = kwargs['yscale']
    else:
        yscale = 1.0
    if 'cxscale' in kwargs:
        cxscale = kwargs['cxscale']
    else:
        cxscale = 1.0

    # Get current position of color bar
    cp = hc.ax.get_position()

    # ToDo: Find better way of determining location
    # Calculate location
    if 'checkSTATS' in option:
        # Taylor diagram
        location = [cp.x0 +xscale*0.8*cp.width, yscale*cp.y0, 
                    cxscale*cp.width/6, cp.height]
    else:
        # target diagram
        location = [cp.x0 +xscale*cp.width, yscale*cp.y0, 
                    cxscale*cp.width/6, cp.height]

    return location



import numpy as np
from . import brier_score

def skill_score_brier(forecast,reference,observed):
    '''
    
    Calculate Brier skill score (BSS) between two variables 
%Calculate non-dimensional skill score (SS) between two variables using
    definition of Murphy (1988)

    Calculates the non-dimensional skill score (SS) difference between two 
    probabilities FORECAST and REFERENCE relative to OBSERVED outcomes. The 
    skill score is calculated using the formula:

    BSS = 1 - BS/BSref
 
    where BS & BSref are the Brier scores for the forecast and reference
    probability of event occurrence
 
    BS    = sum_(n=1)^N (f_n - o_n)^2/N
    BSref = sum_(n=1)^N (r_n - o_n)^2/N
 
    where f is the forecast probabilities, o is the observed probabilities 
    (0 or 1), and N is the total number of values in f & o. Note that f, r, 
    & o must have the same number of values.
 
    Input:
    FORECAST  : forecast probabilities
    REFERENCE : reference probabilities
    OBSERVED  : observed probabilities
 
    Output:
    BSS : Brier skill score

    Reference:
    Glenn W. Brier, 1950: Verification of forecasts expressed in terms 
    of probabilities. Mon. We. Rev., 78, 1-23.
    
    D. S. Wilks, 1995: Statistical Methods in the Atmospheric Sciences. 
    Cambridge Press. 547 pp.

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 15, 2016
    '''

    # Check that dimensions of forecast and reference fields match
    pdims= forecast.shape
    rdims= reference.shape
    if not np.array_equal(pdims,rdims):
        message = 'forecast and reference field dimensions do not' + \
            ' match.\n' + \
            'shape(forecast)= ' + str(pdims) + ', ' + \
            'shape(reference)= ' + str(rdims) + \
            '\nforecast type: ' + str(type(forecast))
        raise ValueError(message)

    #% Calculate skill score
    bss = 1 - brier_score(forecast,observed)/ \
          brier_score(reference,observed)
    return bss



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def pick(columns,
         input_ts='-',
         start_date=None,
         end_date=None,
         round_index=None,
         dropna='no',
         skiprows=None,
         index_type='datetime',
         names=None,
         clean=False):
    """Will pick a column or list of columns from input.

    Can use column names or column numbers.  If using numbers, column
    number 1 is the first data column.

    Parameters
    ----------
    {columns}
    {input_ts}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}

    """
    return tsutils.printiso(
        tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                skiprows=skiprows,
                                                names=names,
                                                index_type=index_type),
                            start_date=start_date,
                            end_date=end_date,
                            pick=columns,
                            round_index=round_index,
                            dropna=dropna,
                            clean=clean))



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def tstopickle(filename,
               input_ts='-',
               columns=None,
               start_date=None,
               end_date=None,
               round_index=None,
               dropna='no',
               skiprows=None,
               index_type='datetime',
               names=None,
               clean=False):
    """Pickle the data into a Python pickled file.

    Can be brought back into Python with 'pickle.load' or 'numpy.load'.
    See also 'tstoolbox read'.

    Parameters
    ----------
    filename : str
         The filename to store the pickled data.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    tsd.to_pickle(filename)



from array import array
import numpy as np
from . import get_target_diagram_axes
from . import get_target_diagram_options
from . import overlay_target_diagram_circles
from . import plot_pattern_diagram_colorbar
from . import plot_pattern_diagram_markers
from . import plot_target_axes

def target_diagram(*args, **kwargs):
    '''
    Plot a target diagram from statistics of different series.

    target_diagram(Bs,RMSDs,RMSDz,keyword=value)

    The first 3 arguments must be the inputs as described below followed by
    keywords in the format OPTION = value. An example call to the function
    would be:

    target_diagram(Bs,RMSDs,RMSDz,markerdisplayed='marker')

    INPUTS:
    Bs    : Bias (B) or Normalized Bias (B*). Plotted along y-axis
            as "Bias".
    RMSDs : unbiased Root-Mean-Square Difference (RMSD') or normalized
            unbiased Root-Mean-Square Difference (RMSD*'). Plotted along
            x-axis as "uRMSD".
    RMSDz : total Root-Mean-Square Difference (RMSD) or other quantities
            (if 'nonRMSDz' == 'on'). Labeled on plot as "RMSD".

    OUTPUTS:
    None.

    LIST OF OPTIONS:
    For an exhaustive list of options to customize your diagram, call the
    function without arguments at a Python command line:
    % python
    >> from tstoolbox.skill_metrics.target_diagram import target_diagram
    >> target_diagram()

    Reference:

    Jolliff, J. K., J. C. Kindle, I. Shulman, B. Penta, M. Friedrichs,
      R. Helber, and R. Arnone (2009), Skill assessment for coupled
      biological/physical models of marine systems, J. Mar. Sys., 76(1-2),
      64-82, doi:10.1016/j.jmarsys.2008.05.014

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Nov 25, 2016
    '''

    # Check for number of arguments
    nargin = len(args)
    Bs, RMSDs, RMSDz = _get_target_diagram_arguments(*args)
    if nargin == 0: return

    # Get options
    option = get_target_diagram_options(**kwargs)

    #  Get axis values for plot
    axes = get_target_diagram_axes(RMSDs,Bs,option)

    # Plot axes for target diagram
    if option['overlay'] == 'off': plot_target_axes(axes)

    # __ Overlay circles
    overlay_target_diagram_circles(option)

    # Plot data points
    lowcase = option['markerdisplayed'].lower()
    if lowcase == 'marker':
        plot_pattern_diagram_markers(RMSDs,Bs,option)
    elif lowcase == 'colorbar':
        plot_pattern_diagram_colorbar(RMSDs,Bs,RMSDz,option)
    else:
        raise ValueError('Unrecognized option: ' +
                         option['markerdisplayed'])

def _get_target_diagram_arguments(*args):
    '''
    Get arguments for target_diagram function.

    Retrieves the arguments supplied to the TARGET_DIAGRAM function as
    arguments and displays the optional arguments if none are supplied.
    Otherwise, tests the first 3 arguments are numeric quantities and
    returns their values.

    INPUTS:
    args : variable-length input argument list

    OUTPUTS:
    Bs    : Bias (B) or Normalized Bias (B*). Plotted along y-axis
            as "Bias".
    RMSDs : unbiased Root-Mean-Square Difference (RMSD') or normalized
            unbiased Root-Mean-Square Difference (RMSD*'). Plotted along
            x-axis as "uRMSD".
    RMSDz : total Root-Mean-Square Difference (RMSD) or other quantities
            (if 'nonRMSDz' == 'on'). Labeled on plot as "RMSD".
    '''

    import numbers

    Bs=[]; RMSDs=[]; RMSDz=[];
    nargin = len(args)
    if nargin == 0:
        # Display options list
        _display_target_diagram_options()
        return Bs, RMSDs, RMSDz
    elif nargin != 3:
        raise ValueError('Must supply 3 arguments.')

    Bs = args[0]
    RMSDs = args[1]
    RMSDz = args[2]

    # Test the above are numeric quantities
    if isinstance(Bs, array):
        Bs = np.array(Bs)
    if isinstance(Bs, numbers.Number):
        Bs = np.array(Bs,ndmin=1)
    if not isinstance(Bs, np.ndarray):
        raise ValueError('Argument Bs is not a numeric array')

    if isinstance(RMSDs, array):
        RMSDs = np.array(RMSDs)
    if isinstance(RMSDs, numbers.Number):
        RMSDs = np.array(RMSDs,ndmin=1)
    if not isinstance(RMSDs, np.ndarray):
        raise ValueError('Argument RMSDs is not a numeric array')

    if isinstance(RMSDz, array):
        RMSDz = np.array(RMSDz)
    if isinstance(RMSDz, numbers.Number):
        RMSDz = np.array(RMSDz,ndmin=1)
    if not isinstance(RMSDz, np.ndarray):
        raise ValueError('Argument RMSDz is not a numeric array')

    return Bs, RMSDs, RMSDz

def _display_target_diagram_options():
    '''
    Displays available options for TARGET_DIAGRAM function.
    '''

    _disp('General options:')
    _dispopt("'overlay'","'on' / 'off' (default): " +
            'Switch to overlay current statistics on target diagram. ' +
            '\n\t\tOnly markers will be displayed.')
    _dispopt("'colormap'","'on'/ 'off' (default): "  +
        "Switch to map color shading of markers to colormap ('on')\n\t\t"  +
        "or min to max range of RMSDz values ('off').\n\t\t"  +
        "Set to same value as option['nonRMSDz'].")
    _disp('')

    _disp('Marker options:')
    _dispopt("'MarkerDisplayed'",
        "'marker' (default): Experiments are represented by individual symbols\n\t\t" +
        "'colorBar': Experiments are represented by a color described " +
        'in a colorbar')
    _disp("OPTIONS when 'MarkerDisplayed' == 'marker'")
    _dispopt("'markerLabel'",'Labels for markers')
    _dispopt("'markerLabelColor'",'Marker label color (Default: black)')
    _dispopt("'markerColor'",'Marker color')
    _dispopt("'markerLegend'","'on' / 'off' (default): Use legend for markers'")
    _dispopt("'markerSize'",'Marker size (Default: 10)')
    _disp("OPTIONS when 'MarkerDisplayed' == 'colorbar'")
    _dispopt("'nonRMSDz'","'on'/ 'off' (default): " +
            'Values in RMSDs do not correspond to total RMS Differences.\n\t\t' +
            '(Used to make range of RMSDs values appear above color bar.)')
    _dispopt("'titleColorBar'",'Title of the colorbar.')
    _disp('')

    _disp('Axes options:')
    _dispopt("'ticks'",'define tick positions ' +
            '(default is that used by axis function)')
    _dispopt("'xtickLabelPos'",'position of the tick labels ' +
            'along the x-axis (empty by default)')
    _dispopt("'ytickLabelPos'",'position of the tick labels ' +
            'along the y-axis (empty by default)')
    _dispopt("'equalAxes'","'on' (default) / 'off': Set axes to be equal")
    _dispopt("'limitAxis'",'Max for the Bias & uRMSD axis')
    _disp('')

    _disp('Diagram options:')
    _dispopt("'alpha'","Blending of symbol face color (0.0 transparent through 1.0 opaque)" +
             "\n\t\t" + "(Default: 1.0)")
    _dispopt("'axismax'",'Maximum for the Bias & uRMSD axis')
    _dispopt("'circles'",'define the radii of circles to draw ' +
            '(default of (maximum RMSDs)*[.7 1], [.7 1] when normalized diagram)')
    _dispopt("'circleLineSpec'",'Circle line specification (default ' +
            "dashed black, '--k')")
    _dispopt("'circleLineWidth'",'Circle line width')
    _dispopt("'obsUncertainty'",'Observational Uncertainty (default of 0)')
    _dispopt("'normalized'","'on' / 'off' (default): normalized target diagram")

def _disp(text):
    print(text)

def _dispopt(optname,optval):
    '''
    Displays option name and values

    This is a support function for the DISPLAY_TARGET_DIAGRAM_OPTIONS function.
    It displays the option name OPTNAME on a line by itself followed by its
    value OPTVAL on the following line.
    '''

    _disp('\t%s' % optname)
    _disp('\t\t%s' % optval)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings
from builtins import range

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def stdtozrxp(input_ts='-',
              columns=None,
              start_date=None,
              end_date=None,
              dropna='no',
              skiprows=None,
              index_type='datetime',
              names=None,
              clean=False,
              round_index=None,
              rexchange=None):
    """Print out data to the screen in a WISKI ZRXP format.

    Parameters
    ----------
    rexchange
        [optional, default is None]

        The REXCHANGE ID to be written into the zrxp header.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    assert len(tsd.columns) == 1, """
*
*   The "stdtozrxp" command can only accept a single
*   'time-series, instead it is seeing {0}.
*
""".format(len(tsd.columns))

    if rexchange:
        print('#REXCHANGE{0}|*|'.format(rexchange))
    for i in range(len(tsd)):
        print(('{0.year:04d}{0.month:02d}{0.day:02d}{0.hour:02d}'
               '{0.minute:02d}{0.second:02d}, {1}').format(
                   tsd.index[i], tsd[tsd.columns[0]][i]))



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def rank(input_ts='-',
         columns=None,
         start_date=None,
         end_date=None,
         dropna='no',
         skiprows=None,
         index_type='datetime',
         names=None,
         clean=False,
         axis=0,
         method='average',
         numeric_only=None,
         na_option='keep',
         ascending=True,
         pct=False,
         print_input=False,
         float_format='%g',
         round_index=None):
    """Compute numerical data ranks (1 through n) along axis.

    Equal values are assigned a rank that is the average of the ranks of those
    values

    Parameters
    ----------
    axis
        [optional, default is 0]

        0 or 'index' for rows. 1 or 'columns' for columns.  Index to direct
        ranking.
    method : str
        [optional, default is 'average']

        +-----------------+--------------------------------+
        | method argument | Description                    |
        +=================+================================+
        | average         | average rank of group          |
        +-----------------+--------------------------------+
        | min             | lowest rank in group           |
        +-----------------+--------------------------------+
        | max             | highest rank in group          |
        +-----------------+--------------------------------+
        | first           | ranks assigned in order they   |
        |                 | appear in the array            |
        +-----------------+--------------------------------+
        | dense           | like 'min', but rank always    |
        |                 | increases by 1 between groups  |
        +-----------------+--------------------------------+

    numeric_only
        [optional, default is None]

        Include only float, int, boolean data. Valid only for DataFrame or
        Panel objects.
    na_option : str
        [optional, default is 'keep']

        +--------------------+--------------------------------+
        | na_option argument | Description                    |
        +====================+================================+
        | keep               | leave NA values where they are |
        +--------------------+--------------------------------+
        | top                | smallest rank if ascending     |
        +--------------------+--------------------------------+
        | bottom             | smallest rank if descending    |
        +--------------------+--------------------------------+

    ascending
        [optional, default is True]

        False ranks by high (1) to low (N)
    pct
        [optional, default is False]

        Computes percentage rank of data.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {print_input}
    {float_format}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)

    # Trying to save some memory
    if print_input:
        otsd = tsd.copy()
    else:
        otsd = pd.DataFrame()

    return tsutils.print_input(print_input,
                               otsd,
                               tsd.rank(axis=axis,
                                        method=method,
                                        numeric_only=numeric_only,
                                        na_option=na_option,
                                        ascending=ascending,
                                        pct=pct),
                               '_rank',
                               float_format=float_format)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def clip(input_ts='-',
         start_date=None,
         end_date=None,
         columns=None,
         dropna='no',
         clean=False,
         skiprows=None,
         index_type='datetime',
         names=None,
         a_min=None,
         a_max=None,
         round_index=None,
         print_input=False):
    """Return a time-series with values limited to [a_min, a_max].

    Parameters
    ---------
    a_min
        [optional, defaults to None]

        All values lower than this will be set to this value.
        Default is None.
    a_max
        [optional, defaults to None]

        All values higher than this will be set to this value.
        Default is None.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {clean}
    {skiprows}
    {index_type}
    {print_input}
    {names}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    for col in tsd.columns:
        if a_min is None:
            try:
                n_min = pd.np.finfo(tsd[col].dtype).min
            except ValueError:
                n_min = pd.np.iinfo(tsd[col].dtype).min
        else:
            n_min = float(a_min)

        if a_max is None:
            try:
                n_max = pd.np.finfo(tsd[col].dtype).max
            except ValueError:
                n_max = pd.np.iinfo(tsd[col].dtype).max
        else:
            n_max = float(a_max)

    return tsutils.print_input(
        print_input, tsd, tsd.clip(n_min, n_max), '_clip')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def aggregate(input_ts='-',
              columns=None,
              start_date=None,
              end_date=None,
              dropna='no',
              clean=False,
              statistic='mean',
              agg_interval='D',
              ninterval=1,
              round_index=None,
              skiprows=None,
              index_type='datetime',
              names=None,
              print_input=False):
    """Take a time series and aggregate to specified frequency.

    Parameters
    ----------
    statistic : str
        [optional, defaults to 'mean']

        'mean', 'sum', 'std', 'max', 'min', 'median', 'first', or 'last'
        to calculate the aggregation.  Can also be a comma separated list of
        statistic methods.
    agg_interval : str
        [optional, defaults to 'D']
        The interval to aggregate the time series.  Any of the PANDAS
        offset codes.

        {pandas_offset_codes}

        There are some deprecated aggregation interval names in
        tstoolbox, DON'T USE!

        +---------------+-----+
        | Instead of... | Use |
        +===============+=====+
        | hourly        | H   |
        +---------------+-----+
        | daily         | D   |
        +---------------+-----+
        | monthly       | M   |
        +---------------+-----+
        | yearly        | A   |
        +---------------+-----+

    ninterval : int
        [optional, defaults to 1]

        The number of agg_interval to use for the aggregation.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {clean}
    {round_index}
    {skiprows}
    {index_type}
    {names}
    {print_input}

    """
    statslist = ['mean',
                 'sum',
                 'std',
                 'max',
                 'min',
                 'median',
                 'first',
                 'last']
    assert statistic in statslist, """
***
*** The statistic option must be one of:
*** {1}
*** to calculate the aggregation.
*** You gave {0}.
***
""".format(statistic, statslist)

    aggd = {'hourly': 'H',
            'daily': 'D',
            'monthly': 'M',
            'yearly': 'A'}
    agg_interval = aggd.get(agg_interval, agg_interval)

    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    methods = statistic.split(',')
    newts = pd.DataFrame()
    for method in methods:
        if method == 'mean':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).mean()
        elif method == 'sum':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).sum()
        elif method == 'std':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).std()
        elif method == 'max':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).max()
        elif method == 'min':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).min()
        elif method == 'median':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).median()
        elif method == 'first':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).first()
        elif method == 'last':
            tmptsd = tsd.resample('{0:d}{1}'.format(ninterval,
                                                    agg_interval)).last()
        tmptsd.rename(columns=lambda x: x + '_' + method, inplace=True)
        newts = newts.join(tmptsd, how='outer')
    return tsutils.print_input(print_input, tsd, newts, '')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def expanding_window(input_ts='-',
                     columns=None,
                     start_date=None,
                     end_date=None,
                     dropna='no',
                     skiprows=None,
                     index_type='datetime',
                     names=None,
                     clean=False,
                     statistic='',
                     min_periods=1,
                     center=False,
                     print_input=False,
                     ):
    """Calculate an expanding window statistic.

    Parameters
    ----------
    statistic : str

        +-----------+----------------------+
        | statistic | Meaning              |
        +===========+======================+
        | corr      | correlation          |
        +-----------+----------------------+
        | count     | count of real values |
        +-----------+----------------------+
        | cov       | covariance           |
        +-----------+----------------------+
        | kurt      | kurtosis             |
        +-----------+----------------------+
        | max       | maximum              |
        +-----------+----------------------+
        | mean      | mean                 |
        +-----------+----------------------+
        | median    | median               |
        +-----------+----------------------+
        | min       | minimum              |
        +-----------+----------------------+
        | skew      | skew                 |
        +-----------+----------------------+
        | std       | standard deviation   |
        +-----------+----------------------+
        | sum       | sum                  |
        +-----------+----------------------+
        | var       | variance             |
        +-----------+----------------------+

    min_periods : int, default 1
        Minimum number of observations in window required to have a value

    center : boolean, default False
        Set the labels at the center of the window.

    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {print_input}
    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              dropna=dropna,
                              clean=clean)

    ntsd = tsd.expanding(min_periods=1,
                         center=False)

    if statistic:
        ntsd = eval('ntsd.{0}()'.format(statistic))

    return tsutils.print_input(print_input,
                               tsd,
                               ntsd,
                               '_expanding_' + statistic)



import os
import xlsxwriter

def write_taylor_stats(filename,data,**kwargs):
    '''
    Write statistics used in a target diagram to an Excel file.
    
    This function writes to an Excel file FILENAME the statistics 
    used to create a Taylor diagram for each of the dictionaries 
    contained in DATA. The first 2 arguments must be the inputs as 
    described below followed by keyword arguments in the format of 
    OPTION = VALUE.
   
    INPUTS:
    filename      : name for statistics Excel file
    data          : a dictionary containing the statistics
    data['sdev']  : Standard deviations (sigma)
    data['crmsd'] : Centered Root Mean Square Difference (CRMSD)
    data['ccoef'] : Correlation Coefficient (r)
   
    OUTPUTS:
        None.
   
    LIST OF OPTIONS:
      A title description for each dictionary (TITLE) can be 
    optionally provided as well as a LABEL for each data point in 
    the diagram.
   
    label = label : label for each data point in target diagram, e.g. 
                    'OC445 (CB)'
    overwrite = boolean : true/false flag to overwrite Excel file
    title = title : title descriptor data set, e.g. 'Expt. 01.0'
  
    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 12, 2016
    '''

    option = get_write_taylor_stats_options(**kwargs)
    
    # Check for existence of file
    if os.path.isfile(filename):
        if option['overwrite']:
            os.remove(filename)
        else:
            ValueError('File already exists: ' + filename)
    
    # Covert data to list if necessary
    if not type(data) is list: data = [data]

    # Write title information to file
    workbook = xlsxwriter.Workbook(filename)
    worksheet = workbook.add_worksheet()
     
    # Write title information to file
    worksheet.write(1, 0, 'Taylor Statistics')

    # Determine number of dictionaries in data variable
    ncell = len(data)
    
    # Write data for each dictionary
    row = 2
    headers = ['Description','Standard Deviation','CRMSD','Correlation Coeff.']
    for i in range(ncell):
        row += 1
        if len(option['title']) > 0:
            worksheet.write(row, 0, option['title'][i])

        # Write column headers
        row += 1
        for j,h in enumerate(headers):
            worksheet.write(row, j, h)

        # Retrieve input values as list
        try: iter(data[i]['sdev'])
        except TypeError:
            sdev = [data[i]['sdev']]
            crmsd = [data[i]['crmsd']]
            ccoef = [data[i]['ccoef']]
        else:
            sdev = data[i]['sdev']
            crmsd = data[i]['crmsd']
            ccoef = data[i]['ccoef']
        ndata = len(sdev)

        # Write each row of data
        row += 1
        for j in range(ndata):
            if len(option['label']) > 0:
                worksheet.write(row, 0, option['label'][j])
    
            worksheet.write(row, 1, sdev[j])
            worksheet.write(row, 2, crmsd[j])
            worksheet.write(row, 3, ccoef[j])
            row += 1

    workbook.close()

def get_write_taylor_stats_options(**kwargs):
    '''
    Get optional arguments for write_taylor_stats function.

    Retrieves the keywords supplied to the WRITE_TARGET_STATS 
    function (**KWARGS), and returns the values in a OPTION dictionary. 
    Default values are assigned to selected optional arguments. The 
    function will terminate with an error if an unrecognized optional 
    argument is supplied.
 
    INPUTS:
    **kwargs : keyword argument list
 
    OUTPUTS:
    option : data structure containing option values.
    option['title']     : title descriptor for data set.
    option['overwrite'] : boolean to overwrite Excel file.
   
    LIST OF OPTIONS:
      A title description for each dataset TITLE can be optionally 
    provided as well as an overwrite option if the file name currently
    exists.
   
    label = label : label for each data point in target diagram, e.g. 
                    'OC445 (CB)'
    overwrite = boolean : true/false flag to overwrite Excel file
    title = title : title descriptor for each data set in data, e.g. 
                   'Expt. 01.0'
  
    Author: Peter A. Rochford
    Acorn Science & Innovation
    prochford@acornsi.com

    Created on Dec 10, 2016

    @author: rochfordp  
    '''
    from . import check_on_off

    nargin = len(kwargs)

    #  Set default parameters
    option = {}
    option['overwrite'] = False
    option['label'] = []
    option['title'] = ''
    if nargin == 0:
        # No options requested, so return with only defaults
        return option

    # Load custom options, storing values in option data structure
        
    # Check for valid keys and values in dictionary
    for optname, optvalue in kwargs.items():
        optname = optname.lower()
        if not optname in option:
            raise ValueError('Unrecognized option: ' + optname)
        else:
            # Replace option value with that from arguments
            option[optname] = optvalue
            
            # Check values for specific options
            if optname == 'overwrite':
                option['overwrite'] = check_on_off(option['overwrite'])

    return option



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def gof(input_ts='-',
        stats='all',
        columns=None,
        start_date=None,
        end_date=None,
        round_index=None,
        clean=False,
        index_type='datetime',
        names=None,
        skiprows=None):
    """Will calculate goodness of fit statistics between two time-series.

    The first time series must be the observed, the second the predicted
    series.  You can only give two time-series.

    Parameters
    ----------
    {input_ts}
    stats
        [optional]

        The statistics that will be presented.
    {columns}
    {start_date}
    {end_date}
    {clean}
    {skiprows}
    {index_type}
    {names}
    {round_index}

    """
    if stats == 'all':
        stats = ['bias', 'rmsd', 'crmsd', 'corrcoef', 'ss']
    else:
        try:
            stats = stats.split(',')
        except AttributeError:
            pass

    # Use dropna='all' to make sure that both have the same missing data.
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna='all',
                              clean=clean)
    if len(tsd.columns) != 2:
        raise ValueError("""
*
*   The gof algorithms work with two time-series only.  You gave {0}.
*
""".format(len(tsd.columns)))

    from .. import skill_metrics as sm
    import pandas as pd

    statval = []

    ref = tsd.iloc[:, 0].values
    pred = tsd.iloc[:, 1].values

    if 'bias' in stats:
        statval.append(['Bias', sm.bias(pred, ref)])

    if 'rmsd' in stats:
        statval.append(['rmsd', sm.rmsd(pred, ref)])

    if 'crmsd' in stats:
        statval.append(['crmsd', sm.centered_rms_dev(pred, ref)])

    if 'corrcoef' in stats:
        statval.append(['corrcoef', pd.np.corrcoef(pred, ref)[0, 1]])

    if 'ss' in stats:
        statval.append(['Skill score (Murphy)',
                        sm.skill_score_murphy(pred, ref)])

    return tsutils.printiso(statval,
                            tablefmt='plain',
                            headers=['Statistic', 'Value'])



import numpy as np

def rmsd(predicted,reference):
    '''
    Calculate root-mean-square deviation (RMSD) between two variables

    Calculates the root-mean-square deviation between two variables 
    PREDICTED and REFERENCE. The RMSD is calculated using the 
    formula:

    RMSD^2 = sum_(n=1)^N [(p_n - r_n)^2]/N
 
    where p is the predicted values, r is the reference values, and
    N is the total number of values in p & r. Note that p & r must
    have the same number of values.
 
    Input:
    PREDICTED : predicted values
    REFERENCE : reference values
 
    Output:
    R : root-mean-square deviation (RMSD)

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com

    Created on Dec 9, 2016
    '''

    # Check that dimensions of predicted and reference fields match
    pdims= predicted.shape
    rdims= reference.shape
    if not np.array_equal(pdims,rdims):
        message = 'predicted and reference field dimensions do not' + \
            ' match.\n' + \
            'shape(predicted)= ' + str(pdims) + ', ' + \
            'shape(reference)= ' + str(rdims) + \
            '\npredicted type: ' + str(type(predicted))
        raise ValueError(message)

    # Calculate the RMSE
    r = np.sqrt(np.sum(np.square(predicted - reference))/len(predicted))

    return r



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def ewm_window(input_ts='-',
               columns=None,
               start_date=None,
               end_date=None,
               dropna='no',
               skiprows=None,
               index_type='datetime',
               names=None,
               clean=False,
               statistic='',
               alpha_com=None,
               alpha_span=None,
               alpha_halflife=None,
               alpha=None,
               min_periods=0,
               adjust=True,
               ignore_na=False,
               print_input=False,
               ):
    """Provides exponential weighted functions.

    Exactly one of center of mass, span, half-life, and alpha must be provided.
    Allowed values and relationship between the parameters are specified in the
    parameter descriptions above; see the link at the end of this section for
    a detailed explanation.

    When adjust is True (default), weighted averages are calculated using
    weights (1-alpha)**(n-1), (1-alpha)**(n-2), . . . , 1-alpha, 1.

    When adjust is False, weighted averages are calculated recursively as:
        weighted_average[0] = arg[0]; weighted_average[i]
        = (1-alpha)*weighted_average[i-1] + alpha*arg[i].

    When ignore_na is False (default), weights are based on absolute positions.
    For example, the weights of x and y used in calculating the final weighted
    average of [x, None, y] are (1-alpha)**2 and 1 (if adjust is True), and
    (1-alpha)**2 and alpha (if adjust is False).

    When ignore_na is True (reproducing pre-0.15.0 behavior), weights are based
    on relative positions. For example, the weights of x and y used in
    calculating the final weighted average of [x, None, y] are 1-alpha and
    1 (if adjust is True), and 1-alpha and alpha (if adjust is False).

    More details can be found at
    http://pandas.pydata.org/pandas-docs/stable/computation.html#exponentially-weighted-windows

    Parameters
    ----------
    statistic : str
        Statistic applied to each window.

        +------+--------------------+
        | corr | correlation        |
        +------+--------------------+
        | cov  | covariance         |
        +------+--------------------+
        | mean | mean               |
        +------+--------------------+
        | std  | standard deviation |
        +------+--------------------+
        | var  | variance           |
        +------+--------------------+

    alpha_com : float, optional
        Specify decay in terms of center of mass, alpha=1/(1+com), for com>=0

    alpha_span : float, optional
        Specify decay in terms of span, alpha=2/(span+1), for span1

    alpha_halflife : float, optional
        Specify decay in terms of half-life, alpha=1-exp(log(0.5)/halflife),
        for halflife>0

    alpha : float, optional
        Specify smoothing factor alpha directly, 0<alpha<=1

    min_periods : int, default 0
        Minimum number of observations in window required to have a value
        (otherwise result is NA).

    adjust : boolean, default True
        Divide by decaying adjustment factor in beginning periods to account
        for imbalance in relative weightings (viewing EWMA as a moving average)

    ignore_na : boolean, default False
        Ignore missing values when calculating weights.

    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {print_input}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              dropna=dropna,
                              clean=clean)

    ntsd = tsd.ewm(alpha_com=alpha_com,
                   alpha_span=alpha_span,
                   alpha_halflife=alpha_halflife,
                   alpha=alpha,
                   min_periods=min_periods,
                   adjust=adjust,
                   ignore_na=ignore_na,
                   )

    if statistic:
        ntsd = eval('ntsd.{0}()'.format(statistic))

    return tsutils.print_input(print_input,
                               tsd,
                               ntsd,
                               '_ewm_' + statistic)



import matplotlib.pyplot as plt
import numpy as np

def plot_taylor_axes(axes, cax, option):
    '''
    Plot axes for Taylor diagram.
    
    Plots the x & y axes for a Taylor diagram using the information 
    provided in the AXES dictionary returned by the 
    GET_TAYLOR_DIAGRAM_AXES function.
    
    INPUTS:
    axes   : data structure containing axes information for target diagram
    cax    : handle for plot axes
    option : data structure containing option values. (Refer to 
             GET_TAYLOR_DIAGRAM_OPTIONS function for more information.)
    option['colcor']       : CORs grid and tick labels color (Default: blue)
    option['colrms']       : RMS grid and tick labels color (Default: green)
    option['colstd']       : STD grid and tick labels color (Default: black)
    option['numberpanels'] : number of panels (quadrants) to use for Taylor
                          diagram
    option['tickrms']      : RMS values to plot gridding circles from 
                          observation point
    option['titlecor']     : title for CORRELATION axis
    option['titlerms']     : title for RMS axis
    option['titlestd']     : title fot STD axis
 
    OUTPUTS:
    ax: returns a list of handles of axis labels
    
    Author: Peter A. Rochford
    Acorn Science & Innovation
    prochford@acornsi.com

    Created on Dec 3, 2016

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    '''
    
    ax = []
    axlabweight = 'bold' 
    if option['numberpanels'] == 1:
        # Single panel
        
        if option['titlestd'] == 'on':
            ttt = plt.ylabel('test', fontsize = 14);
            x = -0.15*axes['rmax']; y = 0.8*axes['rmax'];
            handle = plt.text(x,y,'Standard Deviation', rotation = 90, 
                              color = option['colstd'], 
                              fontweight = axlabweight, 
                              fontsize = plt.get(ttt,'fontsize'),
                              horizontalalignment = 'center')
            ax.append(handle)
        
        if option['titlecor'] == 'on':
            pos1 = 45; DA = 15;
            lab = 'Correlation Coefficient' 
            c = np.fliplr([np.linspace(pos1-DA,pos1+DA,len(lab))])[0]
            dd = 1.1*axes['rmax']
            for ii,ith in enumerate(c): 
                handle = plt.text(dd*np.cos(ith*np.pi/180),
                                   dd*np.sin(ith*np.pi/180),
                                   lab[ii]) 
                handle.set(rotation = ith-90,color = option['colcor'],
                            horizontalalignment = 'center',
                            verticalalignment = 'bottom',
                            fontsize = plt.get(ax[0],'fontsize'),
                            fontweight = axlabweight)
                ax.append(handle) 
        
        if option['titlerms'] == 'on':
            pos1 = option['tickrmsangle']+(180-option['tickrmsangle'])/2; 
            DA = 15; pos1 = 160;
            lab = 'RMSD'
            c = np.fliplr([np.linspace(pos1-DA,pos1+DA,len(lab))])[0]
            
            # Find optimal placement of label
            itick = -1
            ratio = 1.0
            while (ratio > 0.7):
                itick+=1
                ratio = (option['axismax'] - option['tickrms'][itick])/option['axismax']
            dd = 0.7*option['tickrms'][itick] + 0.3*option['tickrms'][itick+1]
            
            # Write label in a circular arc               
            for ii,ith in enumerate(c):
                xtextpos = axes['dx'] + dd*np.cos(ith*np.pi/180) 
                ytextpos = dd*np.sin(ith*np.pi/180) 
                handle = plt.text(xtextpos,ytextpos,lab[ii])
                handle.set(rotation = ith-90, color = option['colrms'], 
                    horizontalalignment = 'center', 
                    verticalalignment = 'top', 
                    fontsize = plt.get(ax[0],'fontsize'),
                    fontweight = axlabweight)
                ax.append(handle)
        
    else:
        # Double panel
    
        if option['titlestd'] == 'on':
            ttt = plt.ylabel('test', fontsize = 14);
            x = 0; y = -0.15*axes['rmax'];
            handle = plt.text(x,y,'Standard Deviation', rotation = 0, 
                              color = option['colstd'], 
                              fontweight = axlabweight, 
                              fontsize = plt.get(ttt,'fontsize'),
                              horizontalalignment = 'center')
            ax.append(handle)

        if option['titlecor'] == 'on':
            pos1 = 90; DA = 25;
            lab = 'Correlation Coefficient' 
            c = np.fliplr([np.linspace(pos1-DA,pos1+DA,len(lab))])[0]
            dd = 1.1*axes['rmax']
            for ii,ith in enumerate(c):
                handle = plt.text(dd*np.cos(ith*np.pi/180),
                                  dd*np.sin(ith*np.pi/180),lab[ii])
                handle.set(rotation = ith-90, color = option['colcor'],
                    horizontalalignment = 'center', 
                    verticalalignment = 'bottom', 
                    fontsize = plt.get(ax[0],'fontsize'),
                    fontweight = axlabweight) 
                ax.append(handle) 
        
        if option['titlerms'] == 'on':
            pos1 = 160; DA = 10;
            lab = 'RMSD' 
            c = np.fliplr([np.linspace(pos1-DA,pos1+DA,len(lab))])[0]
            dd = 1.05*option['tickrms'][0]
            for ii,ith in enumerate(c):
                xtextpos = axes['dx'] + dd*np.cos(ith*np.pi/180) 
                ytextpos = dd*np.sin(ith*np.pi/180) 
                handle = plt.text(xtextpos,ytextpos,lab[ii])
                handle.set(rotation = ith-90, color = option['colrms'],
                    horizontalalignment = 'center', 
                    verticalalignment = 'bottom',
                    fontsize = plt.get(ax[0],'fontsize'),
                    fontweight = axlabweight)
                ax.append(handle)
    
    # VARIOUS ADJUSTMENTS TO THE PLOT:
    cax.set_aspect('equal')
    plt.axis('off')
    plt.gcf().patch.set_facecolor('w')

    # set axis limits
    if option['numberpanels'] == 2:
        axislim = [axes['rmax']*x for x in [-1.15, 1.15, 0, 1.15]]
        plt.axis(axislim) 
        plt.plot([-axes['rmax'], axes['rmax']],[0, 0],
                 color = axes['tc'], linewidth = 2)
        plt.plot([0, 0],[0, axes['rmax']], color = axes['tc'])
    else:
        axislim = [axes['rmax']*x for x in [0, 1.15, 0, 1.15]]
        plt.axis(axislim) 
        plt.plot([0, axes['rmax']],[0, 0], color = axes['tc'], 
                 linewidth = 2) 
        plt.plot([0, 0],[0, axes['rmax']], color = axes['tc'], 
                 linewidth = 2) 

    return ax


#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os.path
import sys
import warnings

import mando

from . import tsutils

from .functions.plot import plot
from .functions.createts import createts
from .functions.filter import filter
from .functions.read import read
from .functions.date_slice import date_slice
from .functions.describe import describe
from .functions.peak_detection import peak_detection
from .functions.convert import convert
from .functions.equation import equation
from .functions.pick import pick
from .functions.stdtozrxp import stdtozrxp
from .functions.tstopickle import tstopickle
from .functions.accumulate import accumulate
from .functions.ewm_window import ewm_window
from .functions.expanding_window import expanding_window
from .functions.rolling_window import rolling_window
from .functions.aggregate import aggregate
from .functions.replace import replace
from .functions.clip import clip
from .functions.add_trend import add_trend
from .functions.remove_trend import remove_trend
from .functions.calculate_fdc import calculate_fdc
from .functions.stack import stack
from .functions.unstack import unstack
from .functions.fill import fill
from .functions.gof import gof
from .functions.dtw import dtw
from .functions.pca import pca
from .functions.normalization import normalization
from .functions.converttz import converttz
from .functions.convert_index_to_julian import convert_index_to_julian
from .functions.convert_index import convert_index
from .functions.pct_change import pct_change
from .functions.rank import rank
from .functions.date_offset import date_offset

warnings.filterwarnings('ignore')


@mando.command()
def about():
    """Display version number and system information."""
    tsutils.about(__name__)


def main():
    """Main function."""
    if not os.path.exists('debug_tstoolbox'):
        sys.tracebacklimit = 0
    mando.main()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def read(filenames,
         force_freq=None,
         append='columns',
         columns=None,
         start_date=None,
         end_date=None,
         dropna='no',
         skiprows=None,
         index_type='datetime',
         names=None,
         clean=False,
         float_format='%g',
         round_index=None,
         how='outer'):
    """Collect time series from a list of pickle or csv files.

    Prints the read in time-series in the tstoolbox standard format.

    Parameters
    ----------
    filenames : str
        List of comma delimited filenames to read time series
        from.
    how : str
        [optional, default is 'outer']

        Use PANDAS concept on how to join the separate DataFrames read
        from each file.  If how='outer' represents the union of the
        time-series, 'inner' is the intersection.
    append : str
        [optional, default is 'columns']

        The type of appending to do.  For "combine" option matching column
        indices will append rows, matching row indices will append columns, and
        matching column/row indices use the value from the first dataset.  You
        can use "row" to force an append along either axis.
    force_freq
        [optional]

        Force this frequency for the files.  Typically you will only want to
        enforce a smaller interval where tstoolbox will insert missing values
        as needed.  WARNING: you may lose data if not careful with this option.
        In general, letting the algorithm determine the frequency should always
        work, but this option will override.  Use PANDAS offset codes.

        {pandas_offset_codes}

    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {float_format}
    {round_index}

    """
    assert append in ['combine', 'rows', 'columns'], """
*
*   The "append" keyword must be "combine", "rows", or "columns".
*   You game me {0}.
*
""".format(append)

    if force_freq is not None:
        dropna = 'no'

    filenames = filenames.split(',')
    result = pd.DataFrame()
    result_list = []
    for index, i in enumerate(filenames):
        tsd = tsutils.common_kwds(
                                  tsutils.read_iso_ts(i,
                                                      skiprows=skiprows,
                                                      names=names,
                                                      index_type=index_type),
                                  start_date=start_date,
                                  end_date=end_date,
                                  pick=columns,
                                  round_index=round_index,
                                  dropna=dropna,
                                  force_freq=force_freq,
                                  clean=clean)
        if append != 'combine':
            result_list.append(tsd)
        else:
            result = result.combine_first(tsd)

    if append != 'combine':
        result = pd.concat(result_list, axis=append)

    result.sort_index(inplace=True)

    return tsutils.printiso(result,
                            float_format=float_format)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def convert(input_ts='-',
            columns=None,
            start_date=None,
            end_date=None,
            dropna='no',
            clean=False,
            skiprows=None,
            index_type='datetime',
            names=None,
            factor=1.0,
            offset=0.0,
            print_input=False,
            round_index=None,
            float_format='%g'):
    """Convert values of a time series by applying a factor and offset.

    See the 'equation' subcommand for a generalized form of this
    command.

    Parameters
    ----------
    factor : float
        [optional, default is 1.0]

        Factor to multiply the time series values.
    offset : float
        [optional, default is 0.0]

        Offset to add to the time series values.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {clean}
    {skiprows}
    {index_type}
    {names}
    {print_input}
    {float_format}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    tmptsd = tsd * factor + offset
    return tsutils.print_input(print_input, tsd, tmptsd, '_convert',
                               float_format=float_format)



import matplotlib.pyplot as plt
import numpy as np

def plot_target_axes(axes):
    '''
    Plot axes for target diagram.
    
    Plots the x & y axes for a target diagram using the information 
    provided in the AXES dictionary returned by the 
    GET_TARGET_DIAGRAM_AXES function.
    
    INPUTS:
    axes   : dictionary containing axes information for target diagram
    
    OUTPUTS:
    None

    Author: Peter A. Rochford
        Symplectic, LLC
        www.thesymplectic.com
        prochford@thesymplectic.com
    '''
    
    xtick = axes['xtick']
    ytick = axes['ytick']
    
    # Draw axis lines
    plt.plot([xtick[0], xtick[-1]], [0, 0], 'k')  # x-axis
    plt.plot([0, 0], [ytick[0], ytick[-1]], 'k')  # y-axis
    
    # Get offsets
    gca = plt.gca()
    Xoff = np.diff(gca.get_xlim())[0] / 40
    Yoff = np.diff(gca.get_ylim())[0] / 40
    
    # Plot new ticks with labels
    y = -2.*Yoff
    for i, x in enumerate(xtick):
        plt.plot([x, x], [0, Yoff], '-k')
        plt.text(x, y, axes['xlabel'][i], horizontalalignment='center')

    x = -1.*Xoff
    for i, y in enumerate(ytick):
        if i > 0:
            plt.plot([Xoff, 0], [y, y], '-k')
        else:
            plt.plot([Xoff, 0], [y - 0.01 * Yoff, y - 0.01 * Yoff], '-k')
            
        plt.text(x, y - 0.5 * Yoff, axes['ylabel'][i], horizontalalignment='right')

    # Label x-axis with text at end of axis
    xpos = xtick[-1] + 3 * xtick[-1] / 30;  ypos = 0
    plt.text(xpos, ypos, 'uRMSD', color='k', horizontalalignment='left')
    
    # Label y-axis with text at end of axis
    xpos = 0;  ypos = ytick[-1] + 3 * ytick[-1] / 30
    plt.text(xpos, ypos, 'Bias', color='k', horizontalalignment='center')
    
    plt.setp(gca, frame_on=False)
    plt.axis('equal')
    plt.axis('off')
    plt.gcf().patch.set_facecolor('w')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings
from builtins import map
from builtins import str

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def unstack(column_names,
            input_ts='-',
            columns=None,
            start_date=None,
            end_date=None,
            round_index=None,
            dropna='no',
            skiprows=None,
            index_type='datetime',
            names=None,
            clean=False):
    """Return the unstack of the input table.

    The unstack command takes the stacked table and converts to a
    standard tstoolbox table.

    From::

      Datetime,Columns,Values
      2000-01-01,TS1,1.2
      2000-01-02,TS1,1.8
      2000-01-03,TS1,1.9
      2000-01-01,TS2,1018.2
      2000-01-02,TS2,1453.1
      2000-01-03,TS2,1683.1
      2000-01-01,TS3,0.0032
      2000-01-02,TS3,0.0002
      2000-01-03,TS3,-0.0004

    To::

      Datetime,TS1,TS2,TS3
      2000-01-01,1.2,1018.2,0.0032
      2000-01-02,1.8,1453.1,0.0002
      2000-01-03,1.9,1683.1,-0.0004

    Parameters
    ----------
    column_names
        The column in the table that holds the column names
        of the unstacked data.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}
    {round_index}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              bestfreq=False)

    try:
        newtsd = tsd.pivot_table(index=tsd.index,
                                 values=tsd.columns.drop(column_names),
                                 columns=column_names,
                                 aggfunc='first')
    except ValueError:
        raise ValueError("""
*
*   Duplicate index (time stamp and '{0}') where found.
*   Found these duplicate indices:
*   {1}
*
""".format(column_names,
           tsd.index.get_duplicates()))

    newtsd.index.name = 'Datetime'

    newtsd.columns = ['_'.join(tuple(map(str, col))).rstrip('_')
                      for col in newtsd.columns.values]

    # Remove weird characters from column names
    newtsd.rename(columns=lambda x: ''.join(
        [i for i in str(x) if i not in '\'" ']))

    newtsd = tsutils.common_kwds(newtsd,
                                 start_date=start_date,
                                 end_date=end_date,
                                 dropna=dropna,
                                 clean=clean,
                                 round_index=round_index)

    return tsutils.printiso(newtsd)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

import pandas as pd
from pandas.tseries.frequencies import to_offset

from .. import tsutils


def index_str_formatter(x):
    if int(x) == x:
        return '{0:d}'.format(int(x))
    return '{0:.10f}'.format(x)


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def convert_index(to,
                  interval=None,
                  epoch='julian',
                  input_ts='-',
                  columns=None,
                  start_date=None,
                  end_date=None,
                  round_index=None,
                  dropna='no',
                  clean=False,
                  index_type='datetime',
                  names=None,
                  skiprows=None):
    """Convert datetime to/from Julian dates from different epochs.

    Parameters
    ----------
    to: str
        One of 'number' or 'datetime'.  If 'number', the source time-series
        should have a datetime index to convert to a number.  If 'datetime',
        source data should be a number and the converted index will be
        datetime.
    interval
        [optional, defaults to None]

        The `interval` parameter defines the unit time.  One of the pandas
        offset codes.  The default of 'None' will set the unit time for all
        defined epochs to daily except 'unix' which will defaults to seconds.

        You can give any smaller unit time than daily for all defined epochs
        except 'unix' which requires an interval less than seconds.

        +------------+--------------------+----------------------+
        | epoch      | `interval`=None    | Available `interval` |
        +============+====================+======================+
        | julian,    | 'D', 'daily',      | 'D', 'daily', or     |
        | reduced,   |                    | smaller              |
        | modified,  |                    |                      |
        | truncated, |                    |                      |
        | dublin,    |                    |                      |
        | cnes,      |                    |                      |
        | ccsds,     |                    |                      |
        | lop,       |                    |                      |
        | lilian,    |                    |                      |
        | rata_die,  |                    |                      |
        | mars_sol   |                    |                      |
        +------------+--------------------+----------------------+
        | unix       | 'S', 'seconds'     | 'S', 'seconds', or   |
        |            |                    | smaller              |
        +------------+--------------------+----------------------+
        | arbitrary  | frequency of input | frequency of input   |
        | epoch      |                    | or smaller           |
        +------------+--------------------+----------------------+

        {pandas_offset_codes}
    epoch : str
        [optional, defaults to 'julian']

        Can be one of, 'julian', 'reduced', 'modified', 'truncated', 'dublin',
        'cnes', 'ccsds', 'lop', 'lilian', 'rata_die', 'mars_sol_date', 'unix',
        or a date and time.

        If supplying a date and time, most formats are recognized, however
        the closer the format is to ISO 8601 the better.  Also should check and
        make sure date was parsed as expected.  If supplying only a date, the
        epoch starts at midnight the morning of that date.

        The 'unix' epoch uses a default `interval` of seconds, and all other
        defined epochs use a default `interval` of 'daily'.

        +-----------+----------------+----------------+-------------+
        | epoch     | Epoch          | Calculation    | Notes       |
        +===========+================+================+=============+
        | julian    | 4713-01-01:12  | JD             |             |
        |           | BCE            |                |             |
        +-----------+----------------+----------------+-------------+
        | reduced   | 1858-11-16:12  | JD -           | [ [1]_ ]    |
        |           |                | 2400000        | [ [2]_ ]    |
        +-----------+----------------+----------------+-------------+
        | modified  | 1858-11-17:00  | JD -           | SAO 1957    |
        |           |                | 2400000.5      |             |
        +-----------+----------------+----------------+-------------+
        | truncated | 1968-05-24:00  | floor (JD -    | NASA 1979,  |
        |           |                | 2440000.5)     | integer     |
        +-----------+----------------+----------------+-------------+
        | dublin    | 1899-12-31:12  | JD -           | IAU 1955    |
        |           |                | 2415020        |             |
        +-----------+----------------+----------------+-------------+
        | cnes      | 1950-01-01:00  | JD -           | CNES        |
        |           |                | 2433282.5      | [ [3]_ ]    |
        +-----------+----------------+----------------+-------------+
        | ccsds     | 1958-01-01:00  | JD -           | CCSDS       |
        |           |                | 2436204.5      | [ [3]_ ]    |
        +-----------+----------------+----------------+-------------+
        | lop       | 1992-01-01:00  | JD -           | LOP         |
        |           |                | 2448622.5      | [ [3]_ ]    |
        +-----------+----------------+----------------+-------------+
        | lilian    | 1582-10-15[13] | floor (JD -    | Count of    |
        |           |                | 2299159.5)     | days of the |
        |           |                |                | Gregorian   |
        |           |                |                | calendar,   |
        |           |                |                | integer     |
        +-----------+----------------+----------------+-------------+
        | rata_die  | 0001-01-01[13] | floor (JD -    | Count of    |
        |           | proleptic      | 1721424.5)     | days of the |
        |           | Gregorian      |                | Common      |
        |           | calendar       |                | Era,        |
        |           |                |                | integer     |
        +-----------+----------------+----------------+-------------+
        | mars_sol  | 1873-12-29:12  | (JD - 2405522) | Count of    |
        |           |                | /1.02749       | Martian     |
        |           |                |                | days        |
        +-----------+----------------+----------------+-------------+
        | unix      | 1970-01-01     | JD - 2440587.5 | seconds     |
        |           | T00:00:00      |                |             |
        +-----------+----------------+----------------+-------------+

        .. [1] . Hopkins, Jeffrey L. (2013). Using Commercial Amateur
           Astronomical Spectrographs, p. 257, Springer Science & Business
           Media, ISBN 9783319014425

        .. [2] . Palle, Pere L., Esteban, Cesar. (2014). Asteroseismology, p.
           185, Cambridge University Press, ISBN 9781107470620

        .. [3] . Theveny, Pierre-Michel. (10 September 2001). "Date Format"
           The TPtime Handbook. Media Lab.

    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {round_index}
    {dropna}
    {clean}
    {skiprows}
    {names}
    {index_type}

    """
    if to == 'datetime':
        index_type = 'number'
    elif to == 'number':
        index_type = 'datetime'
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              round_index=round_index,
                              dropna=dropna,
                              clean=clean)
    allowed = {'julian': lambda x: x,
               'reduced': lambda x: x - 2400000,
               'modified': lambda x: x - 2400000.5,
               'truncated': lambda x: pd.np.floor(x - 2440000.5),
               'dublin': lambda x: x - 2415020,
               'cnes': lambda x: x - 2433282.5,
               'ccsds': lambda x: x - 2436204.5,
               'lop': lambda x: x - 2448622.5,
               'lilian': lambda x: pd.np.floor(x - 2299159.5),
               'rata_die': lambda x: pd.np.floor(x - 1721424.5),
               'mars_sol': lambda x: (x - 2405522) / 1.02749,
               'unix': lambda x: (x - 2440587.5)}

    dailies = ['julian',
               'reduced',
               'modified',
               'truncated',
               'dublin',
               'cnes',
               'ccsds',
               'lop',
               'lilian',
               'rata_die',
               'mars_sol']

    epoch_dates = {'julian': 'julian',
                   'reduced': '1858-11-16T12',
                   'modified': '1858-11-17T00',
                   'truncated': '1968-05-24T00',
                   'dublin': '1899-12-31T12',
                   'cnes': '1950-01-01T00',
                   'ccsds': '1958-01-01T00',
                   'lop': '1992-01-01T00',
                   'lilian': '1582-10-15T00',
                   'rata_die': '0001-01-01T00',
                   'mars_sol': '1873-12-29T12',
                   'unix': 'unix'}

    dt = pd.datetime(2000, 1, 1)
    if epoch == 'unix':
        maxinterval = 'S'
    elif epoch in dailies:
        maxinterval = 'D'
    else:
        maxinterval = tsutils.asbestfreq(tsd).index.freqstr

    if interval is not None:
        if epoch == 'unix':
            if dt + to_offset(interval) > dt + to_offset('S'):
                raise ValueError("""
*
*   The `interval` for the 'unix' epoch must be less than 'S' for seconds.
*
""")
        elif epoch in dailies:
            if dt + to_offset(interval) > dt + to_offset('D'):
                raise ValueError("""
*
*   The `interval` for the '{0}' epoch must be less than 'D' for daily.
*
""".format(epoch))
        else:
            if dt + to_offset(interval) > dt + to_offset(maxinterval):
                raise ValueError("""
*
*   The `interval` for the '{0}' epoch must be less than the interval
*   of the time-series, which is '{1}'.
*
""".format(epoch, maxinterval))

    else:
        if epoch == 'unix':
            interval = 'S'
        elif epoch in dailies:
            interval = 'D'
        else:
            interval = maxinterval

    if to == 'number':
        # Index must be datetime - let's make sure
        tsd.index = pd.to_datetime(tsd.index)

        try:
            frac = to_offset(maxinterval).nanos/to_offset(interval).nanos
        except ValueError:
            frac = 1.0

        try:
            tsd.index = allowed[epoch](tsd.index.to_julian_date())*frac
        except KeyError:
            epoch_date = tsutils.parsedate(epoch)
            if interval in ['M', 'MS']:
                tsd.index = ((tsd.index.year - epoch_date.year)*12 +
                             tsd.index.month - epoch_date.month)
            elif interval in ['W']:
                tsd.index = ((tsd.index.to_julian_date() -
                              epoch_date.to_julian_date())/7.0).astype('i')

        if tsutils.test_cli():
            tsd.index = tsd.index.format(formatter=index_str_formatter)
    elif to == 'datetime':
        tsd.index = pd.to_datetime(tsd.index.values,
                                   origin=epoch_dates.setdefault(epoch, epoch),
                                   unit=interval)
    else:
        raise ValueError("""
*
*   The 'to' argument must be 'number' or 'datetime'.  You gave {0}.
*
""".format(to))

    if names is None:
        tsd.index.name = '{0}_date'.format(epoch)
    return tsutils.printiso(tsd,
                            showindex='always')



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

from . import convert_index


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def convert_index_to_julian(epoch='julian',
                            input_ts='-',
                            columns=None,
                            start_date=None,
                            end_date=None,
                            round_index=None,
                            dropna='no',
                            clean=False,
                            index_type='datetime',
                            names=None,
                            skiprows=None):
    """DEPRECATED use convert_index instead.

    For command line::

        tstoolbox convert_index julian ...

    For Python API::

        from tstoolbox import tstoolbox
        ndf = ntstoolbox.convert_index('julian', ...)
    """
    return convert_index('julian',
                         epoch='julian',
                         columns=columns,
                         input_ts=input_ts,
                         start_date=start_date,
                         end_date=end_date,
                         round_index=round_index,
                         dropna=dropna,
                         clean=clean,
                         skiprows=skiprows,
                         names=names,
                         index_type=index_type)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def pca(input_ts='-',
        columns=None,
        start_date=None,
        end_date=None,
        clean=False,
        skiprows=None,
        index_type='datetime',
        names=None,
        n_components=None,
        round_index=None):
    """Return the principal components analysis of the time series.

    Does not return a time-series.

    Parameters
    ----------
    n_components : int
        [optional, default is None]

        The number of groups to separate the time series into.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {clean}
    {skiprows}
    {index_type}
    {names}
    {round_index}

    """
    from sklearn.decomposition import PCA

    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              round_index=round_index,
                              pick=columns,
                              clean=clean)

    pca = PCA(n_components)
    pca.fit(tsd.dropna(how='any'))
    print(pca.components_)



#!/usr/bin/env python
"""Collection of functions for the manipulation of time series."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import warnings

import mando
from mando.rst_text_formatter import RSTHelpFormatter

from .. import tsutils

warnings.filterwarnings('ignore')


@mando.command(formatter_class=RSTHelpFormatter, doctype='numpy')
@tsutils.doc(tsutils.docstrings)
def describe(input_ts='-',
             columns=None,
             start_date=None,
             end_date=None,
             dropna='no',
             skiprows=None,
             index_type='datetime',
             names=None,
             clean=False,
             transpose=False):
    """Print out statistics for the time-series.

    Parameters
    ----------
    transpose
        [optional, default is False]

        If 'transpose' option is used, will transpose describe output.
    {input_ts}
    {columns}
    {start_date}
    {end_date}
    {dropna}
    {skiprows}
    {index_type}
    {names}
    {clean}

    """
    tsd = tsutils.common_kwds(tsutils.read_iso_ts(input_ts,
                                                  skiprows=skiprows,
                                                  names=names,
                                                  index_type=index_type),
                              start_date=start_date,
                              end_date=end_date,
                              pick=columns,
                              dropna=dropna,
                              clean=clean)
    if transpose is True:
        ntsd = tsd.describe().transpose()
    else:
        ntsd = tsd.describe()

    ntsd.index.name = 'Statistic'
    return tsutils.printiso(ntsd,
                            showindex='always')
